/*=====================================================================

  File:      DMHtmlViewer.cs
  Summary:   Data Mining Web Controls is a library sample intended 
			 to extend the SQL Server Data Mining user experience 
			 to the Web. The library provides a lightweight version 
			 of the Data Mining model viewers. Using Data Mining 
			 Web Controls, complex mining models can be browsed 
			 from any machine that has Microsoft Internet Explorer 
			 installed.
  Date:      Oct 20, 2003

---------------------------------------------------------------------

  This file is part of the Microsoft SQL Server Code Samples.
  Copyright (C) 2003 Microsoft Corporation.  All rights reserved.

This source code is intended only as a supplement to Microsoft
Development Tools and/or on-line documentation.  See these other
materials for detailed information regarding Microsoft code samples.

THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
PARTICULAR PURPOSE.

======================================================= */
namespace Microsoft.AnalysisServices.DataMiningHtmlViewers
{
    using System;
    using System.CodeDom;
    using System.Collections;
    using System.ComponentModel;
    using System.ComponentModel.Design.Serialization;
    using System.Configuration;
    using System.Data;
    using System.Globalization;
    using System.IO;
    using System.Security.Permissions;
    using System.Reflection;
    using System.Web;
    using System.Web.UI;
    using System.Web.UI.WebControls;


    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1813:AvoidUnsealedAttributes")]
    internal class LocCategoryAttribute : CategoryAttribute
    {
        public LocCategoryAttribute(string categoryKey)
            : base(categoryKey)
        {
        }

        protected override string GetLocalizedString(string key)
        {
            string categoryName = null;
            // walk up the cultures until we find one with
            // this key as a resource as our category name
            // if we reach the invariant culture, quit.
            //
            for (CultureInfo culture =
                  CultureInfo.CurrentCulture;
                categoryName == null &&
                  culture != CultureInfo.InvariantCulture;
                culture = culture.Parent)
            {
                categoryName = (string)
                    Properties.Resources.ResourceManager.GetString(key, culture);
            }
            return categoryName;
        }
    }

    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1813:AvoidUnsealedAttributes")]
    internal class LocDescriptionAttribute : DescriptionAttribute
    {
        public LocDescriptionAttribute(string descriptionKey)
            : base(descriptionKey)
        {
        }

        /// <summary>
        /// Override the return of the description text to localize the text
        /// </summary>
        public override string Description
        {
            get
            {
                string strDesc = base.Description;
                return Properties.Resources.ResourceManager.GetString(strDesc);
            }
        }
    }

    


	/// <summary>
    /// DMHtmlViewer provides a lightweight version 
    ///	of the Data Mining model viewers. 
    /// </summary>
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1012:AbstractTypesShouldNotHaveConstructors"), AspNetHostingPermission(SecurityAction.LinkDemand, Level = AspNetHostingPermissionLevel.Minimal)]
    [AspNetHostingPermission(SecurityAction.InheritanceDemand, Level = AspNetHostingPermissionLevel.Minimal)]
    public abstract class DMHtmlViewer : System.Web.UI.WebControls.WebControl
    {
        // Properties for creating a connection string.
        private string server;
        private string catalog;
        private string model;
        private static string urlPrefix;
        public const string DMOlapProvider = "MSOLAP.3";
        private System.Data.OleDb.OleDbConnection connection;

        public DMHtmlViewer()
        {
            server = "localhost";
            catalog = "";
            model = "";
        }

        /// <summary>
        /// Save state as a cumulative array of objects.
        /// </summary>
        /// <returns>Returns an array containing the properties of the control.</returns>
        protected override object SaveViewState()
        {
            object baseState = base.SaveViewState();
            object[] allStates = new object[4];
            allStates[0] = baseState;
            allStates[1] = server;
            allStates[2] = catalog;
            allStates[3] = model;
            return allStates;
        }

        /// <summary>
        /// Set class members to the property values.
        /// </summary>
        /// <param name="savedState">Returns an object containing the state information.</param>
        protected override void LoadViewState(object savedState)
        {
            if (savedState != null)
            {
                object[] myState = (object[])savedState;
                if (myState[0] != null)
                    base.LoadViewState(myState[0]);
                if (myState[1] != null)
                    server = (string)myState[1];
                if (myState[2] != null)
                    catalog = (string)myState[2];
                if (myState[3] != null)
                    model = (string)myState[3];
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="fileName"></param>
        /// <returns></returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1804:RemoveUnusedLocals"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1055:UriReturnValuesShouldNotBeStrings"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        protected virtual string GetClientFileUrl(string fileName)
        {
            try
            {
                if (urlPrefix == null)
                {
                    string location = null;
                    location = "/aspnet_client/{0}/{1}/";

                    if (location == null)
                    {
                        location = String.Empty;
                    }

                    else if (location.IndexOf("{0}") >= 0)
                    {
                        AssemblyName assemblyName = GetType().Assembly.GetName();
                        string assembly = assemblyName.Name.Replace('.', '_').ToLower(CultureInfo.CurrentCulture);
                        string version = assemblyName.Version.ToString().Replace('.', '_');
                        location = String.Format(CultureInfo.CurrentCulture, location, assembly, version);
                    }

                    // If DesignTime get local path
                    if (IsDesignTime())
                    {
                        string wwwroot = Path.Combine(Directory.GetDirectoryRoot(Environment.GetFolderPath(Environment.SpecialFolder.System)), "inetpub\\wwwroot");
                        string ClientFilesUrlPrefix = wwwroot + location.Replace('/', '\\');
                        fileName = fileName.Replace('/', '\\');
                    }
                    else
                    {
                        urlPrefix = location;
                    }

                }
            }
            catch (System.Exception /*ex*/)
            {
                urlPrefix = "/aspnet_client/microsoft_analysisservices_datamininghtmlviewers/9_0_0_0/";
            }

            return urlPrefix + fileName;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        protected virtual bool IsDesignTime()
        {
            return (Context == null);
        }

        [
            LocCategory("DataMiningContent"),
            DefaultValue("localhost"),
            LocDescription("DescASServer")
        ]
        public string Server
        {
            get
            {
                return server;
            }
            set
            {
                server = value;
            }
        }

        [
            LocCategory("DataMiningContent"),
            LocDescription("DescASCatalog")
        ]
        public string Database
        {
            get
            {
                return catalog;
            }
            set
            {
                catalog = value;
            }
        }

        [
            LocCategory("DataMiningContent"),
            LocDescription("DescMiningModel")
        ]
        public string Model
        {
            get
            {
                return model;
            }
            set
            {
                model = value;
                this.OnInit(null);
            }
        }

        [
            Bindable(true),
            LocCategory("DataMiningContent"),
            LocDescription("DescOledbConn")
        ]
        public System.Data.OleDb.OleDbConnection Connection
        {
            get
            {
                return connection;
            }
            set
            {
                connection = value;
            }
        }
    }



}
