/*=====================================================================

  File:      DMDecisionTrees.js
  Summary:   Data Mining Web Controls is a library sample intended 
			 to extend the SQL Server Data Mining user experience 
			 to the Web. The library provides a lightweight version 
			 of the Data Mining model viewers. Using Data Mining 
			 Web Controls, complex mining models can be browsed 
			 from any machine that has Microsoft Internet Explorer 
			 installed.
  Date:      Oct 20, 2003

---------------------------------------------------------------------

  This file is part of the Microsoft SQL Server Code Samples.
  Copyright (C) 2003 Microsoft Corporation.  All rights reserved.

This source code is intended only as a supplement to Microsoft
Development Tools and/or on-line documentation.  See these other
materials for detailed information regarding Microsoft code samples.

THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
PARTICULAR PURPOSE.

======================================================= */

	//Common objects, shared by all the tree controls on the page
	var oDMTreeTooltip = window.createPopup();
	
	// Globaly initialize the scripts for DM Tree Viewer
	DMTreeViewerGlobalInit();
	
	
	var stdDMTreeWidth  = 100;
	var stdDMTreeHeight = 30;
	var stdDMTreeBtnSize  = 7;
	var stdDMTreeBarSize  = 10;
	var stdDMTreeVDelimit = 10;
	var stdDMTreeCommonID = '__dmtree_node';
	var stdDMTreeTextID = '__dmtree_text';
	var stdDMTreeButtonID = '__dmtree_btn';
	var stdDMTreeBarDiv = '__dmtree_bar';
	
    

    //
	function DMTreeViewerGlobalInit()
	{
		// Create the VML namespace.
		if( document.namespaces["vml"] == null )
		{
			document.namespaces.add("vml", "urn:schemas-microsoft-com:vml");
		}
		
		
		// Load the DMTreeViewer.css style sheet.
		ssFound = false;
		for ( i = 0; i < document.styleSheets.length; i++ )
		{
			if( document.styleSheets(i).title == "DMTreeViewer.css")
				ssFound = true;
		}
		if( !ssFound )
		{
		    retSS = document.createStyleSheet();
		    retSS.addRule("vml\\:*", "behavior: url(#default#VML);");
			retSS.title = "DMTreeViewer.css";
		}
	}


	
	// Build the objects.
	function DMTreeBuildNodes(NodeData){
		arRet = new Array();
		argPerItem = 5;
		
		for(i=0;i<NodeData.length/argPerItem;i++){
			arRet[i]=new Object()
			arRet[i].description = NodeData[argPerItem*i];
			arRet[i].parentIndex = NodeData[argPerItem*i + 1];
			arRet[i].support = NodeData[argPerItem*i + 2];
			arRet[i].backgroundColor = NodeData[argPerItem*i + 3];
			arRet[i].textColor = NodeData[argPerItem*i + 4];
			
			arRet[i].children   = new Array();
			arRet[i].expanded  = true;

			if( arRet[i].parentIndex == null )
			{
				// root node
				arRet[i].left = 0;
				arRet[i].top = 0;
			}
			else
			{
				var parent = arRet[arRet[i].parentIndex];
				arRet[i].left = parent.left + stdDMTreeWidth;
				arRet[i].top =  parent.top + stdDMTreeHeight*parent.children.length ;
				parent.children[parent.children.length] = i;
			}
			
		}
		
		
		return arRet
	}



	/*
		Calculate the coordinates of the subtree originated in the 
		given node starting from the top left specified corner.
		Return the width of the current subtree.
	*/
	function DMTreeCalculateNode(strTreeID, nodeID, left, top)
	{
		var arNodes = eval(strTreeID + "arNodes");
		var curNode = arNodes[nodeID];

		
		if( !curNode.expanded || curNode.children.length == 0)
		{
			
			curNode.left = left;
			curNode.top = top;
			iRet = document.all[strTreeID + stdDMTreeCommonID + nodeID].clientHeight + stdDMTreeVDelimit;
			return iRet;
		}
		
		// add one stdDMTreeWidth for this node, and one for spacing
		var iCurLeft = left + document.all[strTreeID + stdDMTreeCommonID + nodeID].clientWidth  + stdDMTreeWidth;
		var iCurTop	 = top;
		var iSubtreeWidth  = 0;
		for( var i = 0; i < arNodes[nodeID].children.length; i ++)
		{
			var childNodeID = curNode.children[i];
			
			var iChildWidth = DMTreeCalculateNode( strTreeID, childNodeID, iCurLeft, iCurTop);
			// iChildWidth == width of the child subtree
			iCurTop	+= iChildWidth + stdDMTreeVDelimit;
		}
		
		curNode.top = (iCurTop + top)/2 - document.all[strTreeID + stdDMTreeCommonID + nodeID].clientHeight/2;
		curNode.left = left;

		iRet = (iCurTop - top);
		return iRet;
	} 



	// Converts 12345px to 123.
	function DMStyleWidthToNumber(styleWidth)
	{
		var iRet = new Number(styleWidth.substr(0, styleWidth.length-2));
		return iRet;
	}

	// Connects a given node to its parent.
	function DMTreeConnectToParent(strTreeID, index)
	{
		var arNodes = eval(strTreeID + "arNodes");
		var fx = new Number(), fy = new Number(), tx= new Number(), ty= new Number();
		if( arNodes[index].parentIndex == null)
			return;
		if( !arNodes[arNodes[index].parentIndex ].expanded )
			return;
			

		var divThis = document.all[strTreeID + stdDMTreeCommonID + index ];
		var divParent = document.all[strTreeID + stdDMTreeCommonID + arNodes[index].parentIndex];
		
		
		fx = DMStyleWidthToNumber(divThis.style.left);
		fy = DMStyleWidthToNumber(divThis.style.top) + stdDMTreeHeight/2;
		
		tx = DMStyleWidthToNumber(divParent.style.left) + divParent.clientWidth + stdDMTreeBtnSize + 4;
		ty = DMStyleWidthToNumber(divParent.style.top) + stdDMTreeHeight/2;

		
		var l = document.createElement("vml:line");
		
		l.strokeweight = eval(strTreeID + "LineStrokeWeight");
		l.strokecolor = eval(strTreeID + "LineStrokeColor");
		l.from = fx + "px, " + fy + "px";
		l.to = tx + "px, " + ty + "px";
		
		document.all[strTreeID + "oContainer"].insertBefore(l);
		
	}
	
	// Recalculates the node coordinates and draws them.
	function DMTreeRecalcLayout(strTreeID)
	{
		var iMaxLevel = 0;
		var arNodes = eval(strTreeID + "arNodes");


		// Delete any existing lines.
		arLines = new Array();
		for( i = 0; i< document.all[strTreeID + "oContainer"].children.length; i++)
			if( document.all[strTreeID + "oContainer"].children[i].tagName == "line" )
				arLines[arLines.length] = document.all[strTreeID + "oContainer"].children[i];
		for( i = 0; i < arLines.length; i++)
			arLines[i].removeNode(true);

		// Recalculate the layouts.
		// DMTreeCalculateNode ( rootID = 0; left = 0; top = 0)
		DMTreeCalculateNode( strTreeID, 0, 0, 0);
		
		// Redraw the tree view.
		for( i = 0; i< arNodes.length; i++)
		{
			document.all[strTreeID + stdDMTreeCommonID + i].style.left = arNodes[i].left;
			document.all[strTreeID + stdDMTreeCommonID + i].style.top = arNodes[i].top;
		}
		
		for( i = 0; i< arNodes.length; i++)
		{
			DMTreeConnectToParent(strTreeID, i);
		}

	}
	
	
	// Populates the AttrValues combo, if that one is found
	function DMTreePopulateAttrValues(strTreeID)
	{
		
		var i;
		var oSelect = document.all[strTreeID + "_attrValues"];
		if( oSelect == null)
		{
			// No combo was found.
			return;
		}
		
		var oOption = document.createElement("OPTION");
		oOption.text="[Total Support]";
		oOption.value="-1";
		oOption.selected = true;
		oSelect.add(oOption);
		
		var arHistoInfo = eval(strTreeID + "arHistogramInfo"); 
		
		for( i=0; i< arHistoInfo.length; i ++)
		{
			var oOption = document.createElement("OPTION");
			oOption.text=arHistoInfo[i];
			oOption.value="" + i;
			oOption.selected = false;
			oSelect.add(oOption);
		}
	}
	
	// 
	function DMTreeInitialize(strTreeID)
	{
	  var i = 0;

	  var arNodes = eval(strTreeID + "arNodes");
	  for( i=0; i< arNodes.length; i ++)
	  {
		var oNode = document.createElement("DIV");

		
		var oTextNode  = document.createElement("DIV");
		var oBarDiv    = document.createElement("DIV");
		
		// Create the main node.
		oNode.id = strTreeID + stdDMTreeCommonID + i;
		oNode.style.position="absolute";
		oNode.style.left=arNodes[i].left;
		oNode.style.top=arNodes[i].top;
		oNode.style.width = stdDMTreeWidth;
		oNode.style.height = stdDMTreeHeight + stdDMTreeBarSize + 4;
		oNode.style.visibility = "visible";
		oNode.style.borderColor = "black";
		oNode.style.borderWidth = 1;
		oNode.style.borderStyle = "solid";
		oNode.style.backgroundColor  = arNodes[i].backgroundColor;
		

		

		// Create the text div node. 
		oTextNode.id = strTreeID + stdDMTreeTextID + i;
		oTextNode.style.position="absolute";
		oTextNode.style.fontSize="75%";
		oTextNode.style.left=1;
		oTextNode.style.top=1;
		oTextNode.style.width=stdDMTreeWidth -2;
		oTextNode.style.height=stdDMTreeHeight-2;
		oTextNode.style.cursor = "hand";
		//oTextNode.title = generateTooltip(i);
		if( eval(strTreeID + "TooltipEvent") == "onclick")
		{
			oTextNode.onclick = DMTreeShowTooltip;
		}
		else
			oTextNode.onmouseover = DMTreeShowTooltip;
		
		oTextNode.treeID = strTreeID;
		oTextNode.onmouseout = DMTreeHideTooltip;

		oTextNode.style.visibility = "inherit";
		oTextNode.style.textOverflow = "ellipsis";
		oTextNode.style.overflow = "hidden";
		oTextNode.style.color = arNodes[i].textColor;
		oTextNode.innerHTML = "<P align='center'>" +  arNodes[i].description + "</P>";


		if( arNodes[i].children.length > 0)
		{
			/*
			var oExpandDiv = document.createElement("SPAN");
			
			oExpandDiv.id = strTreeID + stdDMTreeButtonID + i;
			oExpandDiv.style.position="absolute";
			oExpandDiv.style.left=stdDMTreeWidth;
			oExpandDiv.style.top=(stdDMTreeHeight - stdDMTreeBtnSize)/2;
			oExpandDiv.style.width=stdDMTreeBtnSize;
			oExpandDiv.style.height=stdDMTreeBtnSize;
			oExpandDiv.style.borderColor = "black";
			oExpandDiv.style.borderWidth = 1;
			oExpandDiv.style.borderStyle = "solid";
			oExpandDiv.style.visibility = "inherit";		
			oExpandDiv.innerHTML="-";
			oExpandDiv.onclick = DMTreeExpandCollapse;
			oExpandDiv.treeID = strTreeID;
			oExpandDiv.style.fontSize = "8px";
			oExpandDiv.style.fontFamily = "Verdana";
			oExpandDiv.align = "center";
			oExpandDiv.style.align = "center";
			oExpandDiv.style.cursor = "hand";
			*/
			var oExpandDiv = document.createElement("IMG");
			
			oExpandDiv.id = strTreeID + stdDMTreeButtonID + i;
			oExpandDiv.style.position="absolute";
			oExpandDiv.style.left=stdDMTreeWidth;
			oExpandDiv.style.top=(stdDMTreeHeight - stdDMTreeBtnSize)/2;
			oExpandDiv.style.width=stdDMTreeBtnSize;
			oExpandDiv.style.height=stdDMTreeBtnSize;
			oExpandDiv.style.borderColor = "black";
			oExpandDiv.style.borderWidth = 1;
			oExpandDiv.style.borderStyle = "solid";
			oExpandDiv.style.visibility = "inherit";		
			oExpandDiv.src = DT_CollapsedJPGPath;
			oExpandDiv.onclick = DMTreeExpandCollapse;
			oExpandDiv.treeID = strTreeID;
			oExpandDiv.style.fontSize = "8px";
			oExpandDiv.style.fontFamily = "Verdana";
			oExpandDiv.align = "center";
			oExpandDiv.style.align = "center";
			oExpandDiv.style.cursor = "hand";

			oNode.insertBefore(oExpandDiv);
		}			
		
		
		
		// Create the Bar div
		// Create the text div node 
		oBarDiv.id = strTreeID + stdDMTreeBarDiv + i;
		oBarDiv.style.position="absolute";
		oBarDiv.style.left=2;
		oBarDiv.style.top=stdDMTreeHeight;
		oBarDiv.style.width=stdDMTreeWidth - 4;
		oBarDiv.style.height=stdDMTreeBarSize - 2;
		oBarDiv.style.visibility = "inherit";
		
		iLeft = new Number(0);
		var cHistograms = eval(strTreeID + "cHistograms");
		var arHistograms = eval(strTreeID + "arHistograms");
		var arHistogramColors = eval(strTreeID + "arHistogramColors");
		for( j =0; j< cHistograms; j++)
		{
			iWidth = arHistograms[i*cHistograms + j]*(stdDMTreeWidth - 4)/100;
			oNewDiv = document.createElement("DIV");
			oNewDiv.style.backgroundColor = arHistogramColors[j];
			oNewDiv .style.fontSize = "0pt";
			oNewDiv .style.width = iWidth;
			oNewDiv .style.height = stdDMTreeBarSize - 2;
			oNewDiv .style.border = "none";
			oNewDiv .style.position = "absolute";
			oNewDiv .style.top = 0;
			oNewDiv .style.left = iLeft;
			oNewDiv.style.visibility = "inherit";
			iLeft += iWidth;
			oBarDiv.insertBefore(oNewDiv);
		}
			

		oNode.insertBefore(oTextNode);
		oNode.insertBefore(oBarDiv);
		
		document.all[strTreeID + "oContainer"].insertBefore(oNode);
	 }
	 
	 DMTreeRecalcLayout(strTreeID);
 	 
 	 document.all[strTreeID + "oWrapper"].style.overflow=eval(strTreeID + "ScrollBars");
	 document.all[strTreeID + "oContainer"].style.zoom="1.0";
	 
	 DMTreePopulateAttrValues(strTreeID);
	 
	}
	
    //	
	function DMTreeExpandCollapseNode(strTreeID, index, bExpand)
	{
		var arNodes = eval(strTreeID + "arNodes");
		arNodes[index].expanded = bExpand;
		
		
		ImgElement = document.all[strTreeID + stdDMTreeButtonID + index];
		if( ImgElement != null )
		{
			if( bExpand )
				ImgElement.src = DT_CollapsedJPGPath;
			else 
				ImgElement.src = DT_ExpandedJPGPath;
		}
		
		for( var i = 0; i< arNodes [index].children.length; i++)
		{
			childID = arNodes [index].children[i];
			curChild = document.all[strTreeID + stdDMTreeCommonID + childID];
			
			if( bExpand)
			{
				curChild .style.visibility = "visible";
			}
			else
			{
				curChild .style.visibility = "hidden";
				DMTreeExpandCollapseNode(strTreeID, childID, bExpand);
			}
		}
	}

    //
	function DMTreeExpandCollapse()
	{
		ImgElement = window.event.srcElement;
		
		var strTreeID = ImgElement.treeID;
		var Index = ImgElement.id.substr(strTreeID.length + stdDMTreeButtonID.length);
		
		var arNodes = eval(strTreeID + "arNodes");
		
		if( arNodes [Index].expanded == true)
		{
			arNodes [Index].expanded = false;
			DMTreeExpandCollapseNode(strTreeID, Index, false);
		}
		else
		{
			arNodes [Index].expanded = true;
			DMTreeExpandCollapseNode(strTreeID, Index, true);
		}

		DMTreeRecalcLayout(strTreeID);
	}	

	
	//
	function DMTreeHideTooltip()
	{
		oDMTreeTooltip.hide();
	}
	
	//
	function DMTreeShowTooltip()
	{
		DivElement = window.event.srcElement.parentElement;
		
		
		if( DivElement == null )
			return;
		if( DivElement.id.indexOf(stdDMTreeTextID) < 0 )
			return;
		var strTreeID = DivElement.treeID;
		
		var Index = DivElement.id.substr(strTreeID.length + stdDMTreeTextID.length);

        oDMTreeTooltip.document.body.innerHTML = DMTreeGenerateDescriptionHtml(strTreeID, Index);
        
        
        var popupBody = oDMTreeTooltip.document.body;
		// The following popup object is used only to detect what height the 
		// displayed popup object should be using the scrollHeight property. 
		// This is important because the size of the popup object varies 
		// depending on the length of the definition text. This first 
		// popup object is not seen by the user.
		oDMTreeTooltip.show(0, 0, 400, 0);
		var realHeight = popupBody.scrollHeight;
		// Hides the dimension detector popup object.
		oDMTreeTooltip.hide();
		// Shows the actual popup object with correct height.
		oDMTreeTooltip.show(stdDMTreeWidth, stdDMTreeHeight, 400, realHeight, event.srcElement);
	}
	
	//
	function DMTreeGenerateDescriptionHtml(strTreeID, index)
	{
		var arNodes = eval(strTreeID + "arNodes");
		var curNode = arNodes[index];
		var strRet;
		
		if( curNode == null )
			return "";
		
		var strTooltipBk = eval(strTreeID + "TooltipBackground");
		var strTooltipText = eval(strTreeID + "TooltipTextColor");
		
		var strPrefix;
		var strSuffix;
		
		
		strRet = "<DIV STYLE='color:" + strTooltipText  + "; background:" + strTooltipBk  + "; border:1px solid black; font-size:70%;'>";
		strRet	+= "<CENTER><B>" + curNode.description + "</B></CENTER><br>"
		strRet	+= "<SPAN'> &nbsp;Total Cases: " + curNode.support + "</SPAN><br>"
		strRet	+= "<TABLE align='center' width=95% border=1><TR align = 'center'>";
		
		strPrefix = "<TD style='color:" + strTooltipText + ";' >";
		strSuffix = "</TD>";
		// Add table columns
		strRet	+= strPrefix + "Value" + strSuffix;
		strRet	+= strPrefix + "Cases" + strSuffix;
		strRet	+= strPrefix + "Probability" + strSuffix;
		// Histogram column has fixed width
		strPrefix = "<TD width='110' style='color:" + strTooltipText + ";' >";
		strRet	+= strPrefix + "Histogram" + strSuffix;
		strRet	+= "</TR>";
		
		var cHistograms = eval (strTreeID + "cHistograms");
		var arHistogramInfo = eval (strTreeID + "arHistogramInfo");
		var arHistograms = eval (strTreeID + "arHistograms");
		var arHistogramColors = eval (strTreeID + "arHistogramColors");
		var arProbabilities = eval (strTreeID + "arProbabilities");
		var arCasesSupport = eval (strTreeID + "arCasesSupport");
		
		for( var i = 0; i < cHistograms; i++)
		{
			// Generate the histogram data -- attr value, support, probability.
			strRet	+= "<TR>";
			strPrefix = "<TD STYLE='font-size:70%; color:" + strTooltipText + ";'>";
			strSuffix = "</TD>" 
			strRet	+= strPrefix  + arHistogramInfo[i] + strSuffix;
			strRet	+= strPrefix  + arCasesSupport[index*cHistograms + i] + strSuffix;
			strRet	+= strPrefix  + arProbabilities[index*cHistograms + i] + strSuffix;
			
			strRet	+= "<TD STYLE='font-size:70%'>" 
			
			
			// Generate the histogram bar.
			strRet  += "<DIV STYLE=\"font-size:0%; border:none; height:10;left:5;";
			strRet  += "width:" + arHistograms[index*cHistograms + i] + ";";
			strRet	+= "background:\'" + arHistogramColors[i] + "';";
			strRet	+=	"\"></DIV>";
			strRet  += "</TD>";			
		
			strRet	+= "</TR>\r\n";
		}
		
		
		strRet	+= "</TABLE>";
		
		
		
		strRet += "</DIV>"
		
		return strRet;
		
	}


	// ZoomIn and Out functions.
	function DMTreeZoomIn(strTreeID)
	{
		var fOrigZoom = document.all[strTreeID + "oContainer"].style.zoom;
		document.all[strTreeID + "oContainer"].style.zoom = fOrigZoom*1.25;
	}

	function DMTreeZoomOut(strTreeID)
	{
		var fOrigZoom = document.all[strTreeID + "oContainer"].style.zoom;
		document.all[strTreeID + "oContainer"].style.zoom = fOrigZoom*0.8;
	}




	// Change the background coloring per attribute state.
	function DMTreeDensityFactorChanged(strTreeID)
	{
		var oSelect = document.all[strTreeID + "_attrValues"];
		if( oSelect == null)
		{
			// No combo was found.
			return;
		}
		
		var j = 0;
		var i = 0;
		
		var iState = eval(oSelect.value);
		var min = 0;
		var max = 0;
		
		
		var arNodes = eval(strTreeID + "arNodes");
		var arCasesSupport = eval(strTreeID + "arCasesSupport");
		var arHistograms = eval(strTreeID + "arHistograms");
		var cHistograms = eval(strTreeID + "cHistograms");
		
		if (arCasesSupport.length == 0 )
			return;
		
		// Calculate min/max
		if( iState < 0 )
		{
			// [Total Support]
			min = 0;
			max = 0;
			
			// get the max by summing the first row in arCasesSupport
			for( j = 0; j < cHistograms; j++)
			{
				max = max + arCasesSupport[j];
			}
		}
		else
		{
			// find min and max for the given state in the arCasesSupport
			min	=	arHistograms[iState];
			max	=	arHistograms[iState];
			
			for( i = 1; i < arNodes.length; i++ )
			{
				if	( min > arHistograms[i*cHistograms + iState] )
				{
				 	min = arHistograms[i*cHistograms + iState];
				}

				if	( max < arHistograms[i*cHistograms + iState] )
				{
				 	max = arHistograms[i*cHistograms + iState];
				}
			}
		}
		
		
		// First, cache RMin, RMax...
		var RMin	=	eval(strTreeID + "LowDensity_R");
		var RMax	=	eval(strTreeID + "HighDensity_R");
		var GMin	=	eval(strTreeID + "LowDensity_G");
		var GMax	=	eval(strTreeID + "HighDensity_G");
		var BMin	=	eval(strTreeID + "LowDensity_B");
		var BMax	=	eval(strTreeID + "HighDensity_B");
		
		
		// Now, we have min, max and state. Let's change the node background
		for( i = 0; i < arNodes.length; i++)
		{
			var oNode = document.all[strTreeID + stdDMTreeCommonID + i];
			var oTextNode = document.all[strTreeID + stdDMTreeTextID + i];
			if( oNode == null )
				continue;
			if( oTextNode == null )
				continue;
				

			var dRatio = 1.0;
			var curValue = 0;
			var curNode = arNodes[i];
			
			if( iState < 0 )
				// Calculate ratio between node support and max 
				curValue	=	curNode.support;
			else
				// Calculate ratio between node support and max 
				curValue	=	arHistograms[i*cHistograms + iState];
			
			if( max == 0 )
				dRatio = 0;
			else
				dRatio = curValue/max;
			
			
			
			var newR, newG, newB;
			var strT;
			
			var newR  = Math.round(RMin + dRatio*(RMax - RMin));
			var newG  = Math.round(GMin + dRatio*(GMax - GMin));
			var newB  = Math.round(BMin + dRatio*(BMax - BMin));
			
			var htmlNewCol = "#";
			
			// RGB
			strT = newR.toString(16);
			if( strT.length < 2 )
				htmlNewCol = htmlNewCol + "0";
			htmlNewCol = htmlNewCol + strT;
			
			strT = newG.toString(16);
			if( strT.length < 2 )
				htmlNewCol = htmlNewCol + "0";
			htmlNewCol = htmlNewCol + strT;

			strT = newB.toString(16);
			if( strT.length < 2 )
				htmlNewCol = htmlNewCol + "0";
			htmlNewCol = htmlNewCol + strT;
			
			var htmlNewFore;
			if( (30*newR + 59*newG + 11*newB) > (50*255) )
			{
				htmlNewFore = "#000000";
			}
			else
			{
			 	htmlNewFore = "#ffffff";
			}
			
			
			oNode.style.backgroundColor = htmlNewCol;
			oTextNode.style.color = htmlNewFore;
		}
	}
	
	function getNewColor(R, G, B)
	{
	}