/*============================================================================
  File:    frmMain.cs

  Summary: Contains main form of sample application

           Part of Aggregation Manager 

  Date:    January 2007
------------------------------------------------------------------------------
  This file is part of the Microsoft SQL Server Code Samples.

  Copyright (C) Microsoft Corporation.  All rights reserved.

  This source code is intended only as a supplement to Microsoft
  Development Tools and/or on-line documentation.  See these other
  materials for detailed information regarding Microsoft code samples.

  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Globalization;
using System.Reflection;

using Microsoft.AnalysisServices.AdomdClient;
using Microsoft.AnalysisServices;

namespace AggManager
{
    public partial class MainForm : Form
    {

        Server server;
        private static string TagForLoadingNodes = "LOADING";
        private static string TagDatabases = "Databases";
        private static string TagCubes = "Cubes";
        private static string TagMeasureGroups = "Measure Groups";
        private static string TagAggdesigns = "Aggregation Designs";
        private static string TagNoAggdesign = "No Aggregation Design";

        public const int ImgListMetadataConnectedIndex = 0;
        public const int ImgListMetadataDisconnectedIndex = 1;
        public const int ImgListMetadataFolderIndex = 2;
        public const int ImgListMetadataDatabaseIndex = 3;
        public const int ImgListMetadataCubeIndex = 4;
        public const int ImgListMetadataMeasureGroupIndex = 5;
        public const int ImgListMetadataPartitionIndex = 6;
        public const int ImgListMetadataAggDesIndex = 7;


        string[,] dimAttributes;
        string[] dimNames;
        string[] dimIDs;
        MeasureGroup mgCurrent;

        public MainForm()
        {
            InitializeComponent();

        }

        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main()
        {
            Application.Run(new MainForm());
        }

        private void frmMain_Load(object sender, EventArgs e)
        {

        }

        private void connectButton_Click(object sender, EventArgs e)
        {
            try
            {
                this.Enabled = false;

                //--------------------------------------------------------------------------------
                // Cleanup
                //--------------------------------------------------------------------------------
                this.treeView1.Nodes.Clear();
                if (null != server)
                {
                    this.server.Disconnect();
                }
                else
                {
                    this.server = new Server();
                }

                //--------------------------------------------------------------------------------
                // Connect to Analysis Services server and create tree nodes for Server object.
                //--------------------------------------------------------------------------------
                this.server.Connect(this.connectionStringTextBox.Text);
                TreeNode node = CreateStaticNode(this.treeView1.Nodes, this.server.Name, this.server, ImgListMetadataConnectedIndex );
                CreateNode(node.Nodes, TagDatabases, TagDatabases, ImgListMetadataFolderIndex);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
            finally
            {
                this.Enabled = true;
            }

        }
        private static TreeNode CreateNode(TreeNodeCollection nodes, string text, object obj, int imageIndex)
        {
            TreeNode node = nodes.Add(text, text);
            node.ImageIndex = imageIndex;
            node.SelectedImageIndex = imageIndex;
            node.Tag = obj;
            node.Nodes.Add("... loading ...").Tag = TagForLoadingNodes;
            return node;
        }
        private static TreeNode CreateStaticNode(TreeNodeCollection nodes, string text, object obj, int imageIndex)
        {
            TreeNode node = nodes.Add(text, text );
            node.ImageIndex = imageIndex;
            node.SelectedImageIndex = imageIndex;
            node.Tag = obj;
            return node;
        }

   
        private void treeView1_AfterExpand(object sender, TreeViewEventArgs e)
        {
            //--------------------------------------------------------------------------------
            // Some validations.
            //--------------------------------------------------------------------------------
            if (null == e)
            {
                return;
            }

            TreeNode node = e.Node;
            if (null == node)
            {
                return;
            }

            //--------------------------------------------------------------------------------
            // If the TreeNode being expanded has the information already loaded, we'll do nothing.
            //
            // REMINDER: we are using a special child TreeNode with Tag = "LOADING" to mark
            //           TreeNodes that don't have their children loaded.
            //--------------------------------------------------------------------------------
            TreeNodeCollection nodes = node.Nodes;
            if ((nodes.Count != 1)
                || (nodes[0].Tag != (object)TagForLoadingNodes))
            {
                return;
            }
            nodes.Clear();
            TreeNode nd;
            switch ((string)node.Tag)
            {
                case "Databases":
                    this.Cursor = Cursors.WaitCursor;
                    foreach (Database db in server.Databases)
                    {
                        nd = CreateStaticNode(node.Nodes, db.Name, db, ImgListMetadataDatabaseIndex);
                        CreateNode(nd.Nodes, TagCubes, TagCubes,ImgListMetadataFolderIndex );
                    }
                    this.Cursor = Cursors.Default;
                    break;
                case "Cubes":
                    Database db1 = (Database)node.Parent.Tag;
                    this.Cursor = Cursors.WaitCursor;

                    foreach (Cube cb in db1.Cubes)
                    {
                        nd = CreateStaticNode(node.Nodes, cb.Name, cb, ImgListMetadataCubeIndex);
                        CreateNode(nd.Nodes, TagMeasureGroups, TagMeasureGroups, ImgListMetadataFolderIndex);
                    }
                    this.Cursor = Cursors.Default;
                    break;
                case "Measure Groups":
                    Cube cb1 = (Cube)node.Parent.Tag;

                    this.Cursor = Cursors.WaitCursor;
                    foreach (MeasureGroup mg in cb1.MeasureGroups)
                    {
                        nd = CreateStaticNode(node.Nodes, mg.Name, mg, ImgListMetadataMeasureGroupIndex);
                        CreateNode(nd.Nodes, TagAggdesigns, TagAggdesigns, ImgListMetadataFolderIndex);
                    }
                    this.Cursor = Cursors.Default;
                    break;
                case "Aggregation Designs":
                    MeasureGroup mg1 = (MeasureGroup)node.Parent.Tag;
                    TreeNode ndNoAgg = CreateStaticNode(node.Nodes, TagNoAggdesign, TagNoAggdesign, ImgListMetadataFolderIndex);

                    this.Cursor = Cursors.WaitCursor;

                    foreach (AggregationDesign aggdes in mg1.AggregationDesigns)
                    {
                        CreateStaticNode(node.Nodes, aggdes.Name, aggdes.Name, ImgListMetadataAggDesIndex);
                    }

                    foreach (Partition pt in mg1.Partitions)
                    {
                        if (pt.AggregationDesign == null)
                            CreateStaticNode(ndNoAgg.Nodes, pt.Name, pt, ImgListMetadataPartitionIndex);
                        else
                            CreateStaticNode(node.Nodes[node.Nodes.IndexOfKey(pt.AggregationDesign.Name)].Nodes, pt.Name, pt.Name,  ImgListMetadataPartitionIndex);
                    }
                    this.Cursor = Cursors.Default;
                    break;

                default:
                    break;
            }


        }

        private void treeView1_AfterSelect(object sender, TreeViewEventArgs e)
        {
            listBoxReport.Items.Clear();

            TreeNode nd = e.Node;
            if (nd.Parent == null) return;

            try
            {

                switch ((string)nd.Parent.Tag)
                {
                    case "Databases":
                        break;
                    case "Cubes":
                        break;
                    case "Measure Groups":
                        break;
                    case "Aggregation Designs":
                        listBoxReport.Items.Add("Aggregation count");
                        AggregationDesign aggdes = ((MeasureGroup)nd.Parent.Parent.Tag).AggregationDesigns[(string)nd.Tag];
                        listBoxReport.Items.Add(aggdes.Aggregations.Count);
                       break;

                    default:
                        break;
                }
            }
            catch
            {
            }

        }

        private void cmdAddAggregationDesignToolStripMenuItem_Click(object sender, EventArgs e)
        {
            //Add new Aggregation Design
            AggManager.InputForm form1 = new AggManager.InputForm();

            if (treeView1.SelectedNode.Text == TagNoAggdesign)
                form1.Init((MeasureGroup)treeView1.SelectedNode.Parent.Parent.Tag);
            else
                form1.Init((MeasureGroup)treeView1.SelectedNode.Parent.Tag);

            form1.ShowDialog(this);

            //Refresh nodes tree
            if (treeView1.SelectedNode.Text == TagNoAggdesign)
                treeView1.SelectedNode = treeView1.SelectedNode.Parent.Parent;
            else
                treeView1.SelectedNode = treeView1.SelectedNode.Parent;

            treeView1.SelectedNode.Nodes.RemoveAt(0);
            CreateNode(treeView1.SelectedNode.Nodes, TagAggdesigns, TagAggdesigns, ImgListMetadataFolderIndex);
            treeView1.SelectedNode.Nodes[0].Expand();

            if (!treeView1.SelectedNode.Text.Contains("modified"))
                treeView1.SelectedNode.Text = treeView1.SelectedNode.Text + "-****-modified";
 
        }

   
        private void treeView1_NodeMouseClick(object sender, TreeNodeMouseClickEventArgs e)
        {
            if (e.Button == MouseButtons.Right)
            {

                //force node to be selected
                treeView1.SelectedNode = e.Node;


                Point pt = treeView1.PointToClient(Control.MousePosition);
                try
                {
                    if (e.Node.Text.Contains("modified"))
                        contextMenuStripSave.Show(treeView1, pt);

                    if ((string)e.Node.Tag == "Aggregation Designs" || (string)e.Node.Tag == TagNoAggdesign)
                        contextMenuStripMG.Show(treeView1, pt);
                    else if ((string)e.Node.Parent.Tag == "Aggregation Designs")
                        contextMenuStripAggDes.Show(treeView1, pt);

                    if (((string)e.Node.Parent.Parent.Tag == "Aggregation Designs") && 
                        ((string)e.Node.Parent.Tag != TagNoAggdesign))
                        contextMenuStripPartitionAggs.Show(treeView1, pt);
                    
                }
                catch { }
        }
        }

        private void cmdEditAggDes_Click(object sender, EventArgs e)
        {
            AggManager.EditAggs form1 = new AggManager.EditAggs();
            TreeNode node;
            node = treeView1.SelectedNode;
            
            FillCollections ((MeasureGroup) node.Parent.Parent.Tag);

            form1.Init(node.Name, (MeasureGroup) node.Parent.Parent.Tag , dimAttributes, dimNames, dimIDs);
            form1.ShowDialog(this);

            if (!treeView1.SelectedNode.Parent.Parent.Text.Contains("modified"))
                treeView1.SelectedNode.Parent.Parent.Text = treeView1.SelectedNode.Parent.Parent.Text + "-****-modified";
        }

        private void FillCollections( MeasureGroup mg1)
        {

            if (mg1 == mgCurrent) return;

            mgCurrent = mg1;

            dimAttributes = new string[100, 300];
            dimNames = new string[100];
            dimIDs = new string[100];

            int dimI = 0;
            int attI = 0;

            foreach (MeasureGroupDimension mgDim in mg1.Dimensions)
            {
                dimNames[dimI] = mgDim.CubeDimension.Name;
                dimIDs[dimI] = mgDim.CubeDimensionID;
                attI = 0;

                foreach (CubeAttribute cubeDimAttr in mgDim.CubeDimension.Attributes)
                {
                        dimAttributes[dimI, attI] = cubeDimAttr.AttributeID;
                        attI++;
                }
                dimI++;
            }
        }

        private void cmdAddFromQueryLog_Click(object sender, EventArgs e)
        {

            AggManager.QueryLogForm form1 = new AggManager.QueryLogForm();
            TreeNode node;


            if (treeView1.SelectedNode.Text == TagNoAggdesign)
                node = treeView1.SelectedNode.Parent;
            else
                node = treeView1.SelectedNode;

            FillCollections((MeasureGroup)node.Parent.Tag);
            form1.Init(server, (MeasureGroup)node.Parent.Tag, dimAttributes, dimNames, dimIDs, true , null);

            form1.ShowDialog(this);


            //Refresh the tree 
            treeView1.SelectedNode = node.Parent;
            treeView1.SelectedNode.Nodes.RemoveAt(0);
            CreateNode(treeView1.SelectedNode.Nodes, TagAggdesigns, TagAggdesigns, ImgListMetadataFolderIndex);

            treeView1.SelectedNode.Nodes[0].Expand();
            if (!treeView1.SelectedNode.Text.Contains("modified"))
                treeView1.SelectedNode.Text = treeView1.SelectedNode.Text + "-****-modified";

        }

        private void cmdAddAggregationsFromQueryLogToExisting_Click(object sender, EventArgs e)
        {
            AggManager.QueryLogForm form1 = new AggManager.QueryLogForm();
            
                        
            FillCollections((MeasureGroup)treeView1.SelectedNode.Parent.Parent.Tag);
            form1.Init(server, (MeasureGroup)treeView1.SelectedNode.Parent.Parent.Tag, dimAttributes, dimNames, dimIDs, false, treeView1.SelectedNode.Text);

            form1.ShowDialog(this);

            if (!treeView1.SelectedNode.Parent.Parent.Text.Contains("modified"))
                treeView1.SelectedNode.Parent.Parent.Text = treeView1.SelectedNode.Parent.Parent.Text + "-****-modified";

        }

        private void cmdDeleteAggDes_Click(object sender, EventArgs e)
        {

            TreeNode nd = treeView1.SelectedNode;

            if (MessageBox.Show("Would you like to delete Aggregation design:" + nd.Text + "?", "Delete Aggregation design", MessageBoxButtons.OKCancel) == DialogResult.Cancel)
                return;

            //Delete aggregation design from the measure group

            MeasureGroup mg = (MeasureGroup)nd.Parent.Parent.Tag;

            foreach (Partition pt in mg.Partitions)
            {
                
                if (pt.AggregationDesignID == nd.Text)
                    pt.AggregationDesignID = null;
            }

            mg.AggregationDesigns.Remove(nd.Text);

            //Remove agg design from the tree
            treeView1.SelectedNode = treeView1.SelectedNode.Parent.Parent;
            treeView1.SelectedNode.Nodes.RemoveAt(0);
            CreateNode(treeView1.SelectedNode.Nodes, TagAggdesigns, TagAggdesigns, ImgListMetadataFolderIndex);

            treeView1.SelectedNode.Nodes[0].Expand();
            if (!treeView1.SelectedNode.Text.Contains("modified"))
                treeView1.SelectedNode.Text = treeView1.SelectedNode.Text + "-****-modified";
           
        }


        private void addPartitionsToolStripMenuItem_Click(object sender, EventArgs e)
        {

            AggManager.AddPartitionsForm form1 = new AggManager.AddPartitionsForm();
            TreeNode node;
 
            node = treeView1.SelectedNode;
            
            form1.Init(node.Name, (MeasureGroup) node.Parent.Parent.Tag );
            form1.ShowDialog(this);

            //Refresh the tree 
            treeView1.SelectedNode = treeView1.SelectedNode.Parent.Parent;
            treeView1.SelectedNode.Nodes.RemoveAt(0);
            CreateNode(treeView1.SelectedNode.Nodes, TagAggdesigns, TagAggdesigns, ImgListMetadataFolderIndex);

            treeView1.SelectedNode.Nodes[0].Expand();

            if (!treeView1.SelectedNode.Text.Contains("modified"))
                treeView1.SelectedNode.Text = treeView1.SelectedNode.Text + "-****-modified";

        }


        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void toolStripMenuItemSave_Click(object sender, EventArgs e)
        {
            if (MessageBox.Show("Are you sure you want to save measure group: " + ((MeasureGroup)treeView1.SelectedNode.Tag).Name+ "?", "Save Message", MessageBoxButtons.OKCancel) == DialogResult.Cancel)
                return;
   
            this.Cursor = Cursors.WaitCursor;
            ((MeasureGroup)treeView1.SelectedNode.Tag).Update(UpdateOptions.ExpandFull);
            this.Cursor = Cursors.Default;

            MessageBox.Show( "Measure Group " + ((MeasureGroup)treeView1.SelectedNode.Tag).Name+ " has been saved to the server" );
            treeView1.SelectedNode.Text = treeView1.SelectedNode.Text.Remove(treeView1.SelectedNode.Text.IndexOf("-****-modified") - 1);
        }

        private void toolStripMenuItem3_Click(object sender, EventArgs e)
        {
            AggManager.SaveToFileForm form1 = new AggManager.SaveToFileForm();

            form1.Init((MeasureGroup)treeView1.SelectedNode.Tag);

            form1.ShowDialog(this);
            treeView1.SelectedNode.Text = treeView1.SelectedNode.Text.Remove(treeView1.SelectedNode.Text.IndexOf("-****-modified") - 1);

        }

        private void aggregationSizesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            AggManager.PartitionsAggsForm form1 = new AggManager.PartitionsAggsForm();

            form1.Init((MeasureGroup)treeView1.SelectedNode.Parent.Parent.Parent.Tag, treeView1.SelectedNode.Text);

            form1.ShowDialog(this);

            if (!treeView1.SelectedNode.Parent.Parent.Parent.Text.Contains("modified"))
                treeView1.SelectedNode.Parent.Parent.Parent.Text = treeView1.SelectedNode.Parent.Parent.Parent.Text + "-****-modified";

        }



    }
}