/*=====================================================================

  File:      ClusterViewer.cs
  Summary:   Data Mining Web Controls is a library sample intended 
			 to extend the SQL Server Data Mining user experience 
			 to the Web. The library provides a lightweight version 
			 of the Data Mining model viewers. Using Data Mining 
			 Web Controls, complex mining models can be browsed 
			 from any machine that has Microsoft Internet Explorer 
			 installed.
  Date:      Oct 20, 2003

---------------------------------------------------------------------

  This file is part of the Microsoft SQL Server Code Samples.
  Copyright (C) 2003 Microsoft Corporation.  All rights reserved.

This source code is intended only as a supplement to Microsoft
Development Tools and/or on-line documentation.  See these other
materials for detailed information regarding Microsoft code samples.

THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
PARTICULAR PURPOSE.

======================================================= */
namespace Microsoft.AnalysisServices.DataMiningHtmlViewers
{
    using System;
    using System.CodeDom;
    using System.Collections;
    using System.ComponentModel;
    using System.ComponentModel.Design.Serialization;
    using System.Configuration;
    using System.Data;
    using System.Data.SqlClient;
    using System.Globalization;	// FxCop rule additions
    using System.IO;
    using System.Security.Permissions;
    using System.Reflection;
    using System.Web;
    using System.Web.UI;
    using System.Web.UI.WebControls;

    /// <summary>
    /// The DMClusterViewer control can be used to display the 
    /// content of models created using the Microsoft_Clustering 
    /// Data Mining algorithm. Not as rich as the Cluster viewer 
    /// from the Business Intelligence Workbench, the cluster viewer 
    /// only supports Cluster Characteristics and Cluster Discrimination.
    /// </summary>
    [
    ToolboxData("<{0}:DMClusterViewer runat=server></{0}:DMClusterViewer>")
    ]

    [AspNetHostingPermission(SecurityAction.LinkDemand, Level = AspNetHostingPermissionLevel.Minimal)]
    [AspNetHostingPermission(SecurityAction.InheritanceDemand, Level = AspNetHostingPermissionLevel.Minimal)]
    public class DMClusterViewer : ClusterGridViewer
    {
        internal System.Web.UI.WebControls.DropDownList firstClusterDropDownList;
        internal System.Web.UI.WebControls.Label firstClusterLabel;
        internal System.Web.UI.WebControls.DropDownList secondClusterDropDownList;
        internal System.Web.UI.WebControls.Label errorLabel;
        internal System.Web.UI.WebControls.Label secondClusterLabel;

        /// <summary>
        /// Set the connection string and create a new connection object.
        /// </summary>
        /// <returns>Returns an OleDbConnection object.</returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1024:UsePropertiesWhereAppropriate")]
        protected System.Data.OleDb.OleDbConnection GetConnection()
        {
            try
            {
                System.Data.OleDb.OleDbConnection connection;
                connection = new System.Data.OleDb.OleDbConnection();
                string connectionString;
                connectionString = "Provider=" + DMHtmlViewer.DMOlapProvider + ";Data Source=" + this.Server + ";Initial Catalog=" + this.Database;
                connection.ConnectionString = connectionString;
                connection.Open();

                return connection;
            }
            catch (SqlException se)
            {
                ShowError(se.Message);
                return null;
            }
        }

        /// <summary>
        /// Create controls used by the DMClusterViewer.
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        protected override void RenderChildControls()
        {
            try
            {
                firstClusterLabel = new Label();
                firstClusterLabel.Text = Properties.Resources.StrFirstCluster;
                Controls.Add(firstClusterLabel);
                Controls.Add(new LiteralControl("&nbsp;&nbsp;"));

                firstClusterDropDownList = new DropDownList();
                Controls.Add(firstClusterDropDownList);
                firstClusterDropDownList.SelectedIndexChanged += new EventHandler(this.OnValueChanged);
                firstClusterDropDownList.AutoPostBack = true;
                Controls.Add(new LiteralControl("&nbsp;&nbsp;"));

                secondClusterLabel = new Label();
                secondClusterLabel.Text = Properties.Resources.StrSecondCluster;
                Controls.Add(secondClusterLabel);
                Controls.Add(new LiteralControl("&nbsp;&nbsp;"));

                secondClusterDropDownList = new DropDownList();
                Controls.Add(secondClusterDropDownList);
                secondClusterDropDownList.SelectedIndexChanged += new EventHandler(this.OnValueChanged);
                secondClusterDropDownList.AutoPostBack = true;
                Controls.Add(new LiteralControl("&nbsp;&nbsp;"));

                Controls.Add(new LiteralControl("<br><br>"));

                errorLabel = new Label();
                errorLabel.Text = Properties.Resources.StrError;
                errorLabel.Visible = false;
                errorLabel.ForeColor = System.Drawing.Color.Red;
                Controls.Add(errorLabel);
                Controls.Add(new LiteralControl("&nbsp;&nbsp;"));

                CreateTable();
            }
            catch (Exception ex)
            {
                ShowError(ex.Message);
            }
        }

        /// <summary>
        /// Reinitialize controls to their default settings.
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        protected override void InitializeContent()
        {
            switch (ViewerMode)
            {
                case GridViewerMode.Characteristics:
                    secondClusterLabel.Visible = false;
                    secondClusterDropDownList.Visible = false;
                    break;
                case GridViewerMode.Discrimination:
                    secondClusterLabel.Visible = true;
                    secondClusterDropDownList.Visible = true;
                    break;
            }

            errorLabel.Visible = false;
            if (IsDesignTime())
            {
                try
                {
                    InitializeValues(null);
                }
                catch (System.Exception ex)
                {
                    ShowError(ex.Message);
                }

                string errText = "";
                if (this.Model.Length == 0)
                {
                    errText += Properties.Resources.ErrEmptyModel;
                }

                if (errText.Length > 0)
                {
                    ShowError(errText);
                }
            }
            else
            {
                bool isPostBack = true;
                if (firstClusterDropDownList.Items.Count == 0)
                {
                    isPostBack = false;
                }

                if (!isPostBack)
                {
                    System.Data.OleDb.OleDbConnection connection = null;
                    try
                    {
                        connection = GetConnection();
                        InitializeValues(connection);
                    }
                    catch (System.Exception ex)
                    {
                        ShowError(ex.Message);
                    }
                    finally
                    {
                        if (connection != null)
                        {
                            if (connection.State == ConnectionState.Open)
                                connection.Close();
                        }
                    }
                }
            }
        }

        /// <summary>
        /// Repopulate the grid whenever a new value is picked 
        /// from the list.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        private void OnValueChanged(object sender, System.EventArgs e)
        {
            System.Data.OleDb.OleDbConnection connection = null;

            try
            {
                connection = GetConnection();
                InitializeGrid(connection);
            }
            catch (System.Exception ex)
            {
                ShowError(ex.Message);
            }
            finally
            {
                if (connection != null)
                {
                    if (connection.State == ConnectionState.Open)
                        connection.Close();
                }
            }
            RenderPseudoClusters((sender == firstClusterDropDownList) ? 1 : 2);
        }

        /// <summary>
        /// Reinitialize variables and controls to their 
        /// default settings.
        /// </summary>
        /// <param name="connection">Connection object.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        protected void InitializeValues(System.Data.OleDb.OleDbConnection connection)
        {
            System.Data.OleDb.OleDbCommand cmd = null;
            System.Data.OleDb.OleDbDataReader rdr = null;

            try
            {
                System.Collections.ArrayList arLeftValues = new ArrayList();
                System.Collections.ArrayList arRightValues = new ArrayList();

                if (connection != null)
                {
                    string model = this.Model;
                    model = model.Replace("'", "''");
                    string stmt = "CALL System.Microsoft.AnalysisServices.System.DataMining.Clustering.GetClusters('" + model + "')";

                    cmd = new System.Data.OleDb.OleDbCommand();
                    cmd.CommandText = stmt;
                    cmd.Connection = connection;
                    rdr = cmd.ExecuteReader();

                    firstClusterDropDownList.Items.Clear();
                    secondClusterDropDownList.Items.Clear();

                    while (rdr.Read())
                    {
                        string nameText = rdr.GetValue(0).ToString(); ;
                        string valueText = rdr.GetValue(1).ToString();

                        arLeftValues.Add(new ListItem(nameText, valueText));
                        arRightValues.Add(new ListItem(nameText, valueText));
                    }
                }
                else
                {
                    // Add a couple of design time values.
                    arLeftValues.Add(new ListItem(Properties.Resources.StrSample1, Properties.Resources.StrSample1));
                    arRightValues.Add(new ListItem(Properties.Resources.StrSample2, Properties.Resources.StrSample2));
                }

                int maxLen = 0;
                for (int i = 0; i < arLeftValues.Count; i++)
                {
                    ListItem itm = (ListItem)arLeftValues[i];
                    string nameText = itm.Text;
                    if (nameText.Length > maxLen)
                    {
                        maxLen = nameText.Length;
                    }

                    firstClusterDropDownList.Items.Add(itm);
                    secondClusterDropDownList.Items.Add((ListItem)arRightValues[i]);
                }

                if (maxLen > 0)
                {
                    int CHAR_WIDTH = 5;
                    System.Web.UI.WebControls.Unit u1, u2, u3;

                    u3 = new Unit(firstClusterDropDownList.Style["LEFT"], CultureInfo.CurrentCulture);
                    int start = System.Convert.ToInt32(u3.Value);

                    start = start + maxLen * CHAR_WIDTH + 50;
                    firstClusterLabel.Style["LEFT"] = start.ToString(CultureInfo.CurrentCulture) + "px";
                    firstClusterDropDownList.Width = new Unit(((maxLen + 10) * CHAR_WIDTH + 15), UnitType.Point);

                    u1 = new Unit(secondClusterDropDownList.Style["LEFT"], CultureInfo.CurrentCulture);
                    u2 = new Unit(secondClusterLabel.Style["LEFT"], CultureInfo.CurrentCulture);
                    int delta = System.Convert.ToInt32(u1.Value - u2.Value, CultureInfo.CurrentCulture);

                    secondClusterDropDownList.Style["LEFT"] = (start + delta).ToString(CultureInfo.CurrentCulture) + "px";
                    secondClusterDropDownList.Width = new Unit(((maxLen + 10) * CHAR_WIDTH + 15), UnitType.Point);
                }

                if (firstClusterDropDownList.Items.Count >= 2)
                {
                    firstClusterDropDownList.SelectedIndex = 0;
                    secondClusterDropDownList.SelectedIndex = 1;
                }
                else if (firstClusterDropDownList.Items.Count > 0)
                {
                    firstClusterDropDownList.SelectedIndex = 0;
                    secondClusterDropDownList.SelectedIndex = 0;
                }
                else
                {
                    ShowNote(Properties.Resources.ErrNoClusters);
                    return;
                }
            }
            catch (System.Exception ex)
            {
                ShowError(ex.Message);
            }
            finally
            {
                if (cmd != null)
                {
                    cmd.Dispose();
                }
                if (rdr != null)
                {
                    if (!rdr.IsClosed)
                        rdr.Close();
                }
            }

            RenderPseudoClusters(0);
            InitializeGrid(connection);
        }

        /// <summary>
        /// Reinitialize variables and controls to their 
        /// default settings.
        /// </summary>
        /// <param name="connection">Connection object.</param>
        private void InitializeGrid(System.Data.OleDb.OleDbConnection connection)
        {
            switch (ViewerMode)
            {
                case GridViewerMode.Characteristics:
                    InitializeCharacteristicsGrid(connection);
                    break;
                case GridViewerMode.Discrimination:
                    InitializeDiscriminationGrid(connection);
                    break;
            }
        }

        /// <summary>
        /// Reinitialize variables and controls to their 
        /// default settings.
        /// </summary>
        /// <param name="connection">Connection object.</param>
        private void InitializeDiscriminationGrid(System.Data.OleDb.OleDbConnection connection)
        {
            if (firstClusterDropDownList.SelectedIndex < 0)
                return;
            if (secondClusterDropDownList.SelectedIndex < 0)
                return;


            System.Data.OleDb.OleDbCommand cmd = null;
            System.Data.OleDb.OleDbDataReader rdr = null;

            try
            {
                ResetTable();
                errorLabel.Visible = false;
                SetLeftStateName(firstClusterDropDownList.Items[firstClusterDropDownList.SelectedIndex].Text);
                SetRightStateName(secondClusterDropDownList.Items[secondClusterDropDownList.SelectedIndex].Text);

                if (connection != null)
                {
                    string model = this.Model;
                    string valueText1 = firstClusterDropDownList.Items[firstClusterDropDownList.SelectedIndex].Value.ToString();
                    string valueText2 = secondClusterDropDownList.Items[secondClusterDropDownList.SelectedIndex].Value.ToString();

                    valueText1 = valueText1.Replace("'", "''");
                    valueText2 = valueText2.Replace("'", "''");

                    string stmt = "CALL System.Microsoft.AnalysisServices.System.DataMining.Clustering.GetClusterDiscrimination(";
                    stmt += "'" + model + "', ";
                    stmt += "'" + valueText1 + "', ";
                    stmt += "'" + valueText2 + "', 0)";

                    cmd = new System.Data.OleDb.OleDbCommand();
                    cmd.CommandText = stmt;
                    cmd.Connection = connection;
                    rdr = cmd.ExecuteReader();

                    while (rdr.Read())
                    {
                        string leftCell = rdr.GetValue(0).ToString();
                        string rightCell = rdr.GetValue(1).ToString();

                        if (rightCell.Length == 0)
                        {
                            rightCell = ClusterGridViewer.Missing;
                        }
                        leftCell = leftCell.Replace("&", "&amp;").Replace("<", "&lt;").Replace(">", "&gt;");
                        rightCell = rightCell.Replace("&", "&amp;").Replace("<", "&lt;").Replace(">", "&gt;");

                        double widthDbl;
                        widthDbl = System.Convert.ToDouble(rdr.GetValue(2).ToString(), CultureInfo.CurrentCulture);

                        AddTableRow(leftCell, rightCell, widthDbl);
                    }
                }
                else
                {
                    // Add some Design Time values
                    AddTableRow(Properties.Resources.StrSampleAttribute, Properties.Resources.StrSampleValue, -75);
                    AddTableRow(Properties.Resources.StrSampleAttribute, Properties.Resources.StrSampleValue, 25);
                }
            }
            catch (SqlException se)
            {
                ShowError(se.Message);
            }
            finally
            {
                if (cmd != null)
                {
                    cmd.Dispose();
                }
                if (rdr != null)
                {
                    if (!rdr.IsClosed)
                        rdr.Close();
                }
            }
        }

        /// <summary>
        /// Reinitialize variables and controls to their 
        /// default settings.
        /// </summary>
        /// <param name="connection"></param>
        private void InitializeCharacteristicsGrid(System.Data.OleDb.OleDbConnection connection)
        {
            if (firstClusterDropDownList.SelectedIndex < 0)
                return;

            System.Data.OleDb.OleDbCommand cmd = null;
            System.Data.OleDb.OleDbDataReader rdr = null;

            try
            {
                ResetTable();
                errorLabel.Visible = false;
                SetLeftStateName(firstClusterDropDownList.Items[firstClusterDropDownList.SelectedIndex].Text);

                if (connection != null)
                {
                    string model = this.Model;
                    string valueText1 = firstClusterDropDownList.Items[firstClusterDropDownList.SelectedIndex].Value.ToString();

                    valueText1 = valueText1.Replace("'", "''");

                    string stmt = "CALL System.Microsoft.AnalysisServices.System.DataMining.Clustering.GetClusterCharacteristics(";
                    stmt += "'" + model + "', ";
                    stmt += "'" + valueText1 + "', 0)";

                    cmd = new System.Data.OleDb.OleDbCommand();
                    cmd.CommandText = stmt;
                    cmd.Connection = connection;
                    rdr = cmd.ExecuteReader();

                    while (rdr.Read())
                    {
                        string leftCell = rdr.GetValue(0).ToString();
                        string rightCell = rdr.GetValue(1).ToString();

                        if (rightCell.Length == 0)
                        {
                            rightCell = ClusterGridViewer.Missing;
                        }
                        leftCell = leftCell.Replace("&", "&amp;").Replace("<", "&lt;").Replace(">", "&gt;");
                        rightCell = rightCell.Replace("&", "&amp;").Replace("<", "&lt;").Replace(">", "&gt;");

                        double widthDbl;
                        widthDbl = System.Convert.ToDouble(rdr.GetValue(2).ToString(), CultureInfo.CurrentCulture);

                        AddTableRow(leftCell, rightCell, 100 * widthDbl);
                    }
                }
                else
                {
                    // Add some Design Time values
                    AddTableRow(Properties.Resources.StrSampleAttribute, Properties.Resources.StrSampleValue, -75);
                    AddTableRow(Properties.Resources.StrSampleAttribute, Properties.Resources.StrSampleValue, 25);
                }
            }
            catch (SqlException se)
            {
                ShowError(se.Message);
            }
            finally
            {
                if (cmd != null)
                {
                    cmd.Dispose();
                }
                if (rdr != null)
                {
                    if (!rdr.IsClosed)
                        rdr.Close();
                }
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="currentSelection">The current item selected from the list.</param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        protected void RenderPseudoClusters(int currentSelection)
        {
            try
            {
                if (firstClusterDropDownList.SelectedIndex < 0 ||
                    secondClusterDropDownList.SelectedIndex < 0)
                    return;

                if (currentSelection == 1 && firstClusterDropDownList.Items[firstClusterDropDownList.SelectedIndex].Value.Length == 0)
                {
                    // PCluster just selected on left, so remove 
                    // PCluster from right.
                    ClearPseudoClusters(ref secondClusterDropDownList);
                }
                else if (currentSelection == 2 && secondClusterDropDownList.Items[secondClusterDropDownList.SelectedIndex].Value.Length == 0)
                {
                    // PCluster just selected on right, so remove 
                    // PCluster from left.
                    ClearPseudoClusters(ref firstClusterDropDownList);
                }
                else
                {
                    ClearPseudoClusters(ref firstClusterDropDownList);
                    ClearPseudoClusters(ref secondClusterDropDownList);

                    // No PCluster in the selection. 
                    // Insert PClusters in both lists.
                    string leftCellSel = firstClusterDropDownList.Items[firstClusterDropDownList.SelectedIndex].Text;
                    string rightCellSel = secondClusterDropDownList.Items[secondClusterDropDownList.SelectedIndex].Text;

                    if (ViewerMode == GridViewerMode.Discrimination)
                    {
                        firstClusterDropDownList.Items.Add(new ListItem(Properties.Resources.StrComplementOf + rightCellSel, ""));
                        secondClusterDropDownList.Items.Add(new ListItem(Properties.Resources.StrComplementOf + leftCellSel, ""));
                    }
                    else
                    {
                        firstClusterDropDownList.Items.Insert(0, new ListItem(Properties.Resources.StrPopulationAll, ""));
                    }
                }
            }
            catch (System.Exception ex)
            {
                ShowError(ex.Message);
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="clusterList"></param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1820:TestForEmptyStringsUsingStringLength"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1045:DoNotPassTypesByReference")]
        protected static void ClearPseudoClusters(ref System.Web.UI.WebControls.DropDownList clusterList)
        {
            int index = 0;

            while (index < clusterList.Items.Count)
            {
                if (clusterList.Items[index].Value == "")
                {
                    clusterList.Items.RemoveAt(index);
                }
                else
                {
                    index++;
                }
            }
        }

        /// <summary>
        /// Display error message.
        /// </summary>
        /// <param name="errText">Text of the error message.</param>
        protected void ShowError(string errText)
        {
            errorLabel.Visible = true;
            errorLabel.ForeColor = System.Drawing.Color.Red;
            errorLabel.Text = errText;
        }

        /// <summary>
        /// Display warning message.
        /// </summary>
        /// <param name="errText">Text of the warning message.</param>
        protected void ShowNote(string errText)
        {
            errorLabel.Visible = true;
            errorLabel.ForeColor = System.Drawing.Color.Navy;
            errorLabel.Text = errText;
        }
    }
}
