/*=====================================================================

  File:      NBDiscViewer.cs
  Summary:   Data Mining Web Controls is a library sample intended 
			 to extend the SQL Server Data Mining user experience 
			 to the Web. The library provides a lightweight version 
			 of the Data Mining model viewers. Using Data Mining 
			 Web Controls, complex mining models can be browsed 
			 from any machine that has Microsoft Internet Explorer 
			 installed.
  Date:      Oct 20, 2003

---------------------------------------------------------------------

  This file is part of the Microsoft SQL Server Code Samples.
  Copyright (C) 2003 Microsoft Corporation.  All rights reserved.

This source code is intended only as a supplement to Microsoft
Development Tools and/or on-line documentation.  See these other
materials for detailed information regarding Microsoft code samples.

THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
PARTICULAR PURPOSE.

======================================================= */
namespace Microsoft.AnalysisServices.DataMiningHtmlViewers
{
    using System;
    using System.CodeDom;
    using System.Collections;
    using System.ComponentModel;
    using System.ComponentModel.Design.Serialization;
    using System.Configuration;
    using System.Data;
    using System.Data.SqlClient;
    using System.Globalization;
    using System.IO;
    using System.Reflection;
    using System.Security.Permissions;
    using System.Web;
    using System.Web.UI;
    using System.Web.UI.WebControls;

    /// <summary>
    /// The DMNaiveBayesViewer control can be used to display 
    /// the content of models created using the Microsoft_Naive_Bayes
    /// Data Mining algorithm. The Nave Bayes viewer only supports 
    /// Attribute Characteristics and Discrimination between 
    /// attribute values. 
    /// </summary>
    [
    ToolboxData("<{0}:DMNaiveBayesViewer runat=server></{0}:DMNaiveBayesViewer>")
    ]

    [AspNetHostingPermission(SecurityAction.LinkDemand, Level = AspNetHostingPermissionLevel.Minimal)]
    [AspNetHostingPermission(SecurityAction.InheritanceDemand, Level = AspNetHostingPermissionLevel.Minimal)]
    public class DMNaiveBayesViewer : ClusterGridViewer
    {
        private System.Web.UI.WebControls.Label attributeLabel;
        private System.Web.UI.WebControls.DropDownList firstAttributeValueDropDownList;
        private System.Web.UI.WebControls.Label firstAttributeValueLabel;
        private System.Web.UI.WebControls.DropDownList attributeDropDownList;
        private System.Web.UI.WebControls.DropDownList secondAttributeValueDropDownList;
        private System.Web.UI.WebControls.Label errorLabel;
        private System.Web.UI.WebControls.Label secondAttributeValueLabel;
        private System.Web.UI.WebControls.Table stateSupportTable;
        private bool isShowStateSupport = true;
        private string tooltipStart = "<DIV style='border:1px solid black; font-size:70%; background:Beige'>";
        private string tooltipEnd = "</DIV>";

        [
        LocCategory("DataMiningLayout"),
        DefaultValue(false),
        LocDescription("DescAttrStateSupport")
        ]
        public bool ShowStateSupport
        {
            get
            {
                return isShowStateSupport;
            }
            set
            {
                isShowStateSupport = value;
                ResetContent();
            }
        }

        /// <summary>
        /// Set the connection string and create a new connection object.
        /// </summary>
        /// <returns>Returns an OleDbConnection object.</returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1024:UsePropertiesWhereAppropriate")]
        protected System.Data.OleDb.OleDbConnection GetConnection()
        {
            try
            {
                System.Data.OleDb.OleDbConnection connection;
                connection = new System.Data.OleDb.OleDbConnection();
                string connString;
                connString = "Provider=" + DMHtmlViewer.DMOlapProvider + ";Data Source=" + this.Server + ";Initial Catalog=" + this.Database;
                connection.ConnectionString = connString;
                connection.Open();
                return connection;
            }
            catch (SqlException se)
            {
                ShowError(se.Message);
                return null;
            }
        }

        /// <summary>
        /// Create controls used by the DMNaiveBayesViewer.
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        protected override void RenderChildControls()
        {
            try
            {
                attributeLabel = new Label();
                attributeLabel.Text = "Attribute:";
                Controls.Add(attributeLabel);
                Controls.Add(new LiteralControl("&nbsp;&nbsp;"));

                attributeDropDownList = new DropDownList();
                attributeDropDownList.SelectedIndexChanged += new EventHandler(this.OnAttributeChanged);
                attributeDropDownList.AutoPostBack = true;
                Controls.Add(attributeDropDownList);

                stateSupportTable = new Table();
                Controls.Add(stateSupportTable);

                Controls.Add(new LiteralControl("<br>"));

                firstAttributeValueLabel = new Label();
                firstAttributeValueLabel.Text = Properties.Resources.StrValue1;
                Controls.Add(firstAttributeValueLabel);
                Controls.Add(new LiteralControl("&nbsp;&nbsp;"));

                firstAttributeValueDropDownList = new DropDownList();
                Controls.Add(firstAttributeValueDropDownList);
                firstAttributeValueDropDownList.SelectedIndexChanged += new EventHandler(this.OnValueChanged);
                firstAttributeValueDropDownList.AutoPostBack = true;
                Controls.Add(new LiteralControl("&nbsp;&nbsp;"));

                secondAttributeValueLabel = new Label();
                secondAttributeValueLabel.Text = Properties.Resources.StrValue2;
                Controls.Add(secondAttributeValueLabel);
                Controls.Add(new LiteralControl("&nbsp;&nbsp;"));

                secondAttributeValueDropDownList = new DropDownList();
                Controls.Add(secondAttributeValueDropDownList);
                secondAttributeValueDropDownList.SelectedIndexChanged += new EventHandler(this.OnValueChanged);
                secondAttributeValueDropDownList.AutoPostBack = true;
                Controls.Add(new LiteralControl("&nbsp;&nbsp;"));

                Controls.Add(new LiteralControl("<br><br>"));

                errorLabel = new Label();
                errorLabel.Text = Properties.Resources.StrError;
                errorLabel.Visible = false;
                errorLabel.ForeColor = System.Drawing.Color.Red;
                Controls.Add(errorLabel);
                Controls.Add(new LiteralControl("&nbsp;&nbsp;"));

                CreateTable();
            }
            catch (System.Exception ex)
            {
                ShowError(ex.Message);
            }
        }

        /// <summary>
        /// Reinitialize controls to their default settings.
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        protected override void InitializeContent()
        {
            stateSupportTable.Visible = isShowStateSupport;

            switch (ViewerMode)
            {
                case GridViewerMode.Characteristics:
                    secondAttributeValueLabel.Visible = false;
                    secondAttributeValueDropDownList.Visible = false;
                    break;
                case GridViewerMode.Discrimination:
                    secondAttributeValueLabel.Visible = true;
                    secondAttributeValueDropDownList.Visible = true;
                    break;
            }

            errorLabel.Visible = false;
            if (IsDesignTime())
            {
                try
                {
                    InitializeAttributes(null);
                }
                catch (System.Exception ex)
                {
                    ShowError(ex.Message);
                }

                string errText = "";
                if (this.Model.Length == 0)
                {
                    errText += Properties.Resources.ErrEmptyModel;
                }

                if (errText.Length > 0)
                {
                    ShowError(errText);
                }
            }
            else
            {
                bool isPostBack = true;
                if (attributeDropDownList.Items.Count == 0)
                {
                    isPostBack = false;
                }

                if (!isPostBack)
                {
                    System.Data.OleDb.OleDbConnection connection = null;

                    try
                    {
                        connection = GetConnection();
                        InitializeAttributes(connection);
                        connection.Close();
                    }
                    catch (System.Exception ex)
                    {
                        ShowError(ex.Message);
                    }
                    finally
                    {
                        if (connection != null)
                        {
                            if (connection.State == ConnectionState.Open)
                                connection.Close();
                        }
                    }
                }
            }
        }

        /// <summary>
        /// Repopulate the grid whenever a new value is picked 
        /// from the list.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        private void OnAttributeChanged(object sender, System.EventArgs e)
        {
            System.Data.OleDb.OleDbConnection connection = null;

            try
            {
                connection = GetConnection();
                InitializeValues(connection);
                connection.Close();
            }
            catch (System.Exception ex)
            {
                ShowError(ex.Message);
            }
            finally
            {
                if (connection != null)
                {
                    if (connection.State == ConnectionState.Open)
                        connection.Close();
                }
            }
        }

        /// <summary>
        /// Repopulate the grid whenever a new value is picked 
        /// from the list.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        private void OnValueChanged(object sender, System.EventArgs e)
        {
            System.Data.OleDb.OleDbConnection connection = null;

            try
            {
                connection = GetConnection();
                InitializeGrid(connection);
            }
            catch (System.Exception ex)
            {
                ShowError(ex.Message);
            }
            finally
            {
                if (connection != null)
                {
                    if (connection.State == ConnectionState.Open)
                        connection.Close();
                }
            }
        }

        /// <summary>
        /// Reinitialize variables and controls to their 
        /// default settings.
        /// </summary>
        /// <param name="connection">Connection object.</param>
        protected void InitializeAttributes(System.Data.OleDb.OleDbConnection connection)
        {
            System.Data.OleDb.OleDbDataReader rdr = null;
            System.Data.OleDb.OleDbCommand cmd = null;

            try
            {
                System.Collections.ArrayList arAttr = new ArrayList();
                System.Collections.ArrayList arIDs = new ArrayList();

                if (connection != null)
                {
                    string modelText = this.Model.Replace("'", "''");
                    string stmt = "CALL System.Microsoft.AnalysisServices.System.DataMining.NaiveBayes.GetPredictableAttributes('" + modelText + "')";

                    cmd = new System.Data.OleDb.OleDbCommand();
                    cmd.CommandText = stmt;
                    cmd.Connection = connection;
                    rdr = cmd.ExecuteReader();

                    while (rdr.Read())
                    {
                        string attrTextName = rdr.GetString(0);
                        string idText = rdr.GetString(1);
                        arAttr.Add(attrTextName);
                        arIDs.Add(idText);
                    }
                }
                else
                {
                    // Set design time sample attributes.
                    arAttr.Add(Properties.Resources.StrSampleAttribute);
                    arIDs.Add(Properties.Resources.StrSampleAttribute);
                }

                int maxLen = 0;
                attributeDropDownList.Items.Clear();

                for (int i = 0; i < arAttr.Count; i++)
                {
                    if (maxLen < ((string)arAttr[i]).Length)
                        maxLen = ((string)arAttr[i]).Length;
                    System.Web.UI.WebControls.ListItem itm = new System.Web.UI.WebControls.ListItem((string)arAttr[i], (string)arIDs[i]);
                    attributeDropDownList.Items.Add(itm);
                }

                int CHAR_WIDTH = 6;
                if (attributeDropDownList.Items.Count > 0)
                {
                    attributeDropDownList.Width = new Unit((maxLen * CHAR_WIDTH + 5), UnitType.Point);
                    attributeDropDownList.SelectedIndex = 0;

                    InitializeValues(connection);
                }
                else
                {
                    ShowNote(Properties.Resources.ErrNoPredictableAttrs);
                }
            }
            catch (SqlException se)
            {
                ShowError(se.Message);
            }
            finally
            {
                if (rdr != null)
                {
                    if (!rdr.IsClosed)
                        rdr.Close();
                }
                if (cmd != null)
                {
                    cmd.Dispose();
                }
            }
        }

        /// <summary>
        /// Reinitialize variables and controls to their 
        /// default settings.
        /// </summary>
        /// <param name="connection">Connection object</param>
        protected void InitializeStateSupport(System.Data.OleDb.OleDbConnection connection)
        {
            if (connection == null)
            {
                return;
            }

            System.Data.OleDb.OleDbDataReader rdr = null;
            System.Data.OleDb.OleDbCommand cmd = null;

            try
            {
                string modelText = this.Model;
                string attrText = attributeDropDownList.Items[attributeDropDownList.SelectedIndex].Value.ToString(CultureInfo.CurrentCulture);
                modelText = modelText.Replace("'", "''");
                attrText = attrText.Replace("'", "''");
                string stmt = "CALL System.Microsoft.AnalysisServices.System.DataMining.NaiveBayes.GetAttributeHistogram('" + modelText + "', '" + attrText + "')";

                cmd = new System.Data.OleDb.OleDbCommand();
                cmd.CommandText = stmt;
                cmd.Connection = connection;
                rdr = cmd.ExecuteReader();

                ResetStateSupportTable();

                System.Collections.ArrayList arNames = new ArrayList();
                System.Collections.ArrayList arValues = new ArrayList();
                double dTotalSupport = 0;

                while (rdr.Read())
                {
                    string valueText = rdr.GetValue(0).ToString();
                    if (valueText.Length == 0)
                    {
                        valueText = ClusterGridViewer.Missing;
                    }
                    string supportText = rdr.GetValue(1).ToString();
                    arNames.Add(valueText);
                    double dSupp = System.Convert.ToDouble(supportText, CultureInfo.CurrentCulture);
                    dTotalSupport += dSupp;
                    arValues.Add(dSupp);
                }

                for (int i = 0; i < arNames.Count; i++)
                {
                    System.Web.UI.WebControls.TableRow row = new System.Web.UI.WebControls.TableRow();
                    System.Web.UI.WebControls.TableCell cell;


                    cell = new TableCell();
                    cell.Text = ((string)arNames[i]).Replace("&", "&amp;").Replace("<", "&lt;").Replace(">", "&gt;");
                    row.Cells.Add(cell);

                    double dWidth = 100 * (double)arValues[i] / dTotalSupport;
                    int width = System.Convert.ToInt32(dWidth);

                    cell = new TableCell();
                    cell.HorizontalAlign = HorizontalAlign.Left;
                    string cellText;
					cellText = " &nbsp;" + System.Convert.ToInt32((double)arValues[i], CultureInfo.CurrentCulture);
                    cell.Text = cellText;
					cell.ToolTip = System.Convert.ToString(arValues[i], CultureInfo.CurrentCulture);
                    row.Cells.Add(cell);

					cell = new TableCell();
					cell.HorizontalAlign = HorizontalAlign.Left;
					cellText = "<DIV STYLE=\"font-size:0%; border:none; height:10px;left:5px;";
					cellText += "width:" + width.ToString(CultureInfo.CurrentCulture) + "px;";
					cellText += "background-color:" + System.Drawing.ColorTranslator.ToHtml(BarColor) + ";";
					cellText += "\"></DIV>";
					cell.Text = cellText;
					cell.ToolTip = System.Convert.ToString(arValues[i], CultureInfo.CurrentCulture);
					row.Cells.Add(cell);


                    stateSupportTable.Rows.Add(row);
                }
            }
            catch (SqlException se)
            {
                ShowError(se.Message);
            }
            finally
            {
                if (rdr != null)
                {
                    if (!rdr.IsClosed)
                        rdr.Close();
                }
                if (cmd != null)
                {
                    cmd.Dispose();
                }
            }
        }

        /// <summary>
        /// Reinitialize variables and controls to their 
        /// default settings.
        /// </summary>
        /// <param name="connection">Connection object.</param>
        protected void InitializeValues(System.Data.OleDb.OleDbConnection connection)
        {
            if (attributeDropDownList.SelectedIndex < 0)
                return;

            System.Data.OleDb.OleDbDataReader rdr = null;
            System.Data.OleDb.OleDbCommand cmd = null;

            try
            {
                System.Collections.ArrayList arLeftValues = new ArrayList();
                System.Collections.ArrayList arRightValues = new ArrayList();

                if (connection != null)
                {
                    string modelText = this.Model;
                    string attrText = attributeDropDownList.Items[attributeDropDownList.SelectedIndex].Value.ToString(CultureInfo.CurrentCulture);
                    modelText = modelText.Replace("'", "''");
                    attrText = attrText.Replace("'", "''");
                    string stmt = "CALL System.Microsoft.AnalysisServices.System.DataMining.NaiveBayes.GetAttributeValues('" + modelText + "', '" + attrText + "')";

                    cmd = new System.Data.OleDb.OleDbCommand();
                    cmd.CommandText = stmt;
                    cmd.Connection = connection;
                    rdr = cmd.ExecuteReader();

                    firstAttributeValueDropDownList.Items.Clear();
                    secondAttributeValueDropDownList.Items.Clear();

                    while (rdr.Read())
                    {
                        string name;
                        string valueText = rdr.GetValue(0).ToString();

                        name = valueText;
                        if (valueText.Length == 0)
                        {
                            name = ClusterGridViewer.Missing;
                        }


                        arLeftValues.Add(new ListItem(name, valueText));
                        arRightValues.Add(new ListItem(name, valueText));
                    }

                }
                else
                {
                    // Add a couple of design time values.
                    arLeftValues.Add(new ListItem(Properties.Resources.StrSample1, Properties.Resources.StrSample1));
                    arRightValues.Add(new ListItem(Properties.Resources.StrSample2, Properties.Resources.StrSample2));
                }

                int maxLen = 0;
                for (int i = 0; i < arLeftValues.Count; i++)
                {
                    ListItem itm = (ListItem)arLeftValues[i];
                    string name = itm.Text;
                    if (name.Length > maxLen)
                    {
                        maxLen = name.Length;
                    }

                    firstAttributeValueDropDownList.Items.Add(itm);
                    secondAttributeValueDropDownList.Items.Add((ListItem)arRightValues[i]);
                }

                if (maxLen > 0)
                {
                    int CHAR_WIDTH = 5;
                    System.Web.UI.WebControls.Unit u1, u2, u3;

                    u3 = new Unit(firstAttributeValueDropDownList.Style["LEFT"], CultureInfo.CurrentCulture);
                    int start = System.Convert.ToInt32(u3.Value, CultureInfo.CurrentCulture);

                    start = start + maxLen * CHAR_WIDTH + 50;
                    firstAttributeValueLabel.Style["LEFT"] = start.ToString(CultureInfo.CurrentCulture) + "px";
                    firstAttributeValueDropDownList.Width = new Unit((maxLen * CHAR_WIDTH + 15), UnitType.Point);

                    u1 = new Unit(secondAttributeValueDropDownList.Style["LEFT"], CultureInfo.CurrentCulture);
                    u2 = new Unit(secondAttributeValueLabel.Style["LEFT"], CultureInfo.CurrentCulture);
                    int delta = System.Convert.ToInt32(u1.Value - u2.Value, CultureInfo.CurrentCulture);

                    secondAttributeValueDropDownList.Style["LEFT"] = (start + delta).ToString(CultureInfo.CurrentCulture) + "px";
                    secondAttributeValueDropDownList.Width = new Unit((maxLen * CHAR_WIDTH + 15), UnitType.Point);
                }

                if (firstAttributeValueDropDownList.Items.Count > 2)
                {
                    firstAttributeValueDropDownList.SelectedIndex = 1;
                    secondAttributeValueDropDownList.SelectedIndex = 2;
                }
                else if (firstAttributeValueDropDownList.Items.Count > 1)
                {
                    firstAttributeValueDropDownList.SelectedIndex = 0;
                    secondAttributeValueDropDownList.SelectedIndex = 1;
                }
                else if (firstAttributeValueDropDownList.Items.Count > 0)
                {
                    firstAttributeValueDropDownList.SelectedIndex = 0;
                    secondAttributeValueDropDownList.SelectedIndex = 0;
                }
                else
                {
                    ShowNote("No states for this attribute were found in the model.\n Most likely, the attribute was ignored as statistically irrelevant.");
                    return;
                }

                InitializeGrid(connection);
            }
            catch (SqlException se)
            {
                ShowError(se.Message);
            }
            finally
            {
                if (rdr != null)
                {
                    if (!rdr.IsClosed)
                        rdr.Close();
                }
                if (cmd != null)
                {
                    cmd.Dispose();
                }
            }
        }


        /// <summary>
        /// Reinitialize variables and controls to their 
        /// default settings.
        /// </summary>
        /// <param name="connection">Connection object.</param>
        private void InitializeGrid(System.Data.OleDb.OleDbConnection connection)
        {
            InitializeStateSupport(connection);

            switch (ViewerMode)
            {
                case GridViewerMode.Characteristics:
                    InitializeCharacteristicsGrid(connection);
                    break;
                case GridViewerMode.Discrimination:
                    InitializeDiscriminationGrid(connection);
                    break;
            }
        }

        /// <summary>
        /// Reinitialize variables and controls to their 
        /// default settings.
        /// </summary>
        /// <param name="connection">Connection object.</param>
        private void InitializeDiscriminationGrid(System.Data.OleDb.OleDbConnection connection)
        {
            if (attributeDropDownList.SelectedIndex < 0)
                return;
            if (firstAttributeValueDropDownList.SelectedIndex < 0)
                return;
            if (secondAttributeValueDropDownList.SelectedIndex < 0)
                return;

            System.Data.OleDb.OleDbDataReader rdr = null;
            System.Data.OleDb.OleDbCommand cmd = null;

            try
            {
                ResetTable();
                errorLabel.Visible = false;
                SetLeftStateName(firstAttributeValueDropDownList.Items[firstAttributeValueDropDownList.SelectedIndex].Text);
                SetRightStateName(secondAttributeValueDropDownList.Items[secondAttributeValueDropDownList.SelectedIndex].Text);

                if (connection != null)
                {
                    string modelText = this.Model;
                    string attrText = attributeDropDownList.Items[attributeDropDownList.SelectedIndex].Value.ToString(CultureInfo.CurrentCulture);
                    string valueText1 = firstAttributeValueDropDownList.Items[firstAttributeValueDropDownList.SelectedIndex].Value.ToString(CultureInfo.CurrentCulture);
                    string valueText2 = secondAttributeValueDropDownList.Items[secondAttributeValueDropDownList.SelectedIndex].Value.ToString(CultureInfo.CurrentCulture);

                    string name1 = firstAttributeValueDropDownList.Items[firstAttributeValueDropDownList.SelectedIndex].Text.ToString(CultureInfo.CurrentCulture);
                    string name2 = secondAttributeValueDropDownList.Items[secondAttributeValueDropDownList.SelectedIndex].Text.ToString(CultureInfo.CurrentCulture);

                    string valueType1 = "1";
                    string valueType2 = "1";

                    if (valueText1.Length == 0)
                    {
                        valueType1 = "0";
                    }

                    if (valueText2.Length == 0)
                    {
                        valueType2 = "0";
                    }
                    name1 = name1.Replace("'", "''");
                    name2 = name2.Replace("'", "''");

                    string stmt = "CALL System.Microsoft.AnalysisServices.System.DataMining.NaiveBayes.GetAttributeDiscrimination(";
                    stmt += "'" + modelText + "', ";
                    stmt += "'" + attrText + "', ";
                    stmt += "'" + name1 + "', ";
                    stmt += valueType1 + ", ";
                    stmt += "'" + name2 + "', ";
                    stmt += valueType2 + ", 0.0)";

                    cmd = new System.Data.OleDb.OleDbCommand();
                    cmd.CommandText = stmt;
                    cmd.Connection = connection;
                    rdr = cmd.ExecuteReader();

                    // Prepare name1 and name2 for HTML rendering
                    name1 = name1.Replace("&", "&amp;").Replace("<", "&lt;").Replace(">", "&gt;");
                    name2 = name2.Replace("&", "&amp;").Replace("<", "&lt;").Replace(">", "&gt;");

                    while (rdr.Read())
                    {
                        string leftText = rdr.GetValue(0).ToString();
                        string rightText = rdr.GetValue(1).ToString();

                        if (rightText.Length == 0)
                        {
                            rightText = ClusterGridViewer.Missing;
                        }
                        leftText = leftText.Replace("&", "&amp;").Replace("<", "&lt;").Replace(">", "&gt;");
                        rightText = rightText.Replace("&", "&amp;").Replace("<", "&lt;").Replace(">", "&gt;");


                        double dWidth;
                        dWidth = System.Convert.ToDouble(rdr.GetValue(2).ToString(), CultureInfo.CurrentCulture);
                        string tooltipText = tooltipStart;

                        tooltipText += "Score:   ";
                        tooltipText += System.Convert.ToInt32(dWidth, CultureInfo.CurrentCulture).ToString(CultureInfo.CurrentCulture);
                        tooltipText += "<br>";
                        tooltipText += "Attribute:   ";
                        tooltipText += leftText.Replace("&", "&amp;").Replace("<", "&lt;").Replace(">", "&gt;");
                        tooltipText += "<br><br>";

                        if (rdr.FieldCount > 6)
                        {
                            // Using extra info
                            string formatText =
                                "<table align=center border=1>" +
                                " <thead>" +
                                "    <td>&nbsp;</td>" +
                                "    <td align=center style='font-size:70%;'><B>{0}</B></td>" +
                                "    <td align=center style='font-size:70%;'><B>{1}</B></td>" +
                                " </thead>" +
                                " <tr>" +
                                "    <td align='left' style='font-size:70%;' >{2}</td>" +
                                "    <td align='center' style='font-size:70%;' >{3}</td>" +
                                "    <td align='center' style='font-size:70%;' >{4}</td>" +
                                " </tr>" +
                                " <tr>" +
                                "    <td align='left' style='font-size:70%;' >NOT {2}</td>" +
                                "    <td align='center' style='font-size:70%;' >{5}</td>" +
                                "    <td align='center' style='font-size:70%;' >{6}</td>" +
                                " </tr>" +
                                "</table>";

                            string tableName;
                            tableName = string.Format(CultureInfo.CurrentCulture,
                                formatText,
                                name1, name2,
                                rightText,
                                rdr.GetValue(3).ToString(), rdr.GetValue(4).ToString(),
                                rdr.GetValue(5).ToString(), rdr.GetValue(6).ToString());

                            tooltipText += tableName;
                            tooltipText += "<br>";
                        }
                        tooltipText += tooltipEnd;
                        AddTableRow(leftText, rightText, dWidth, tooltipText);
                    }
                }
                else
                {
                    // Add some design time values.
                    AddTableRow(Properties.Resources.StrSampleAttribute, Properties.Resources.StrSampleValue, -75);
                    AddTableRow(Properties.Resources.StrSampleAttribute, Properties.Resources.StrSampleValue, 25);
                }
            }
            catch (SqlException se)
            {
                ShowError(se.Message);
            }
            finally
            {
                if (rdr != null)
                {
                    if (!rdr.IsClosed)
                        rdr.Close();
                }
                if (cmd != null)
                {
                    cmd.Dispose();
                }
            }
        }

        /// <summary>
        /// Reinitialize variables and controls to their 
        /// default settings.
        /// </summary>
        /// <param name="connection">Connection object.</param>
        private void InitializeCharacteristicsGrid(System.Data.OleDb.OleDbConnection connection)
        {
            if (attributeDropDownList.SelectedIndex < 0)
                return;

            if (firstAttributeValueDropDownList.SelectedIndex < 0)
                return;

            System.Data.OleDb.OleDbDataReader rdr = null;
            System.Data.OleDb.OleDbCommand cmd = null;

            try
            {
                ResetTable();
                errorLabel.Visible = false;
                SetLeftStateName(firstAttributeValueDropDownList.Items[firstAttributeValueDropDownList.SelectedIndex].Text);

                if (connection != null)
                {
                    string modelText = this.Model;
                    string attrText = attributeDropDownList.Items[attributeDropDownList.SelectedIndex].Value.ToString(CultureInfo.CurrentCulture);
                    string valueText1 = firstAttributeValueDropDownList.Items[firstAttributeValueDropDownList.SelectedIndex].Value.ToString(CultureInfo.CurrentCulture);

                    string name1 = firstAttributeValueDropDownList.Items[firstAttributeValueDropDownList.SelectedIndex].Text.ToString(CultureInfo.CurrentCulture);

                    string valueType1 = "1";

                    if (valueText1.Length == 0)
                    {
                        valueType1 = "0";
                    }


                    valueText1 = valueText1.Replace("'", "''");

                    string stmt = "CALL System.Microsoft.AnalysisServices.System.DataMining.NaiveBayes.GetAttributeCharacteristics(";
                    stmt += "'" + modelText + "', ";
                    stmt += "'" + attrText + "', ";
                    stmt += "'" + name1 + "', ";
                    stmt += valueType1 + ", 0.0)";

                    cmd = new System.Data.OleDb.OleDbCommand();
                    cmd.CommandText = stmt;
                    cmd.Connection = connection;
                    rdr = cmd.ExecuteReader();

                    while (rdr.Read())
                    {
                        string leftText = rdr.GetValue(0).ToString();
                        string rightText = rdr.GetValue(1).ToString();
                        if (rightText.Length == 0)
                        {
                            rightText = ClusterGridViewer.Missing;
                        }
                        leftText = leftText.Replace("&", "&amp;").Replace("<", "&lt;").Replace(">", "&gt;");
                        rightText = rightText.Replace("&", "&amp;").Replace("<", "&lt;").Replace(">", "&gt;");

                        double dWidth;
                        dWidth = 100 * System.Convert.ToDouble(rdr.GetValue(2).ToString(), CultureInfo.CurrentCulture);

                        string tooltipText = tooltipStart;

                        tooltipText += "&nbsp; Score:   ";
                        tooltipText += System.Convert.ToInt32(dWidth, CultureInfo.CurrentCulture).ToString(CultureInfo.CurrentCulture);
                        tooltipText += "<br>";

                        if (rdr.FieldCount > 3)
                        {
                            tooltipText += "&nbsp; " + Properties.Resources.StrSupport + ": ";
                            tooltipText += rdr.GetValue(3).ToString();
                            tooltipText += "<br>";
                        }
                        tooltipText += tooltipEnd;

                        AddTableRow(leftText, rightText, dWidth, tooltipText);
                    }
                }
                else
                {
                    // Add some design time values.
                    AddTableRow(Properties.Resources.StrSampleAttribute, Properties.Resources.StrSampleValue, -75);
                    AddTableRow(Properties.Resources.StrSampleAttribute, Properties.Resources.StrSampleValue, 25);
                }
            }
            catch (SqlException se)
            {
                ShowError(se.Message);
            }
            finally
            {
                if (rdr != null)
                {
                    if (!rdr.IsClosed)
                        rdr.Close();
                }
                if (cmd != null)
                {
                    cmd.Dispose();
                }
            }
        }

        /// <summary>
        /// Display error message.
        /// </summary>
        /// <param name="errText">Text of the error message.</param>
        protected void ShowError(string errText)
        {
            errorLabel.Visible = true;
            errorLabel.ForeColor = System.Drawing.Color.Red;
            errorLabel.Text = errText;
        }

        /// <summary>
        /// Display warning message.
        /// </summary>
        /// <param name="errText">Text of the warning message.</param>
        protected void ShowNote(string errText)
        {
            errorLabel.Visible = true;
            errorLabel.ForeColor = System.Drawing.Color.Navy;
            errorLabel.Text = errText;
        }

        /// <summary>
        /// 
        /// </summary>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        protected void ResetStateSupportTable()
        {
            try
            {
                stateSupportTable.Rows.Clear();

                System.Web.UI.WebControls.TableRow row = new System.Web.UI.WebControls.TableRow();
                System.Web.UI.WebControls.TableHeaderCell hdrCell;

                hdrCell = new TableHeaderCell();
                hdrCell.BackColor = System.Drawing.Color.LightGray;
                hdrCell.Font.Bold = true;
				hdrCell.Text = "&nbsp;&nbsp;&nbsp;&nbsp;" + Properties.Resources.StrValue  + "&nbsp;&nbsp;&nbsp;&nbsp;";
                row.Cells.Add(hdrCell);

                hdrCell = new TableHeaderCell();
                hdrCell.BackColor = System.Drawing.Color.LightGray;
                hdrCell.Font.Bold = true;
				hdrCell.Width = new Unit(100);
				hdrCell.Text = Properties.Resources.StrSupport;//"&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;Support&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;";
                row.Cells.Add(hdrCell);

				hdrCell = new TableHeaderCell();
				hdrCell.BackColor = System.Drawing.Color.LightGray;
				hdrCell.Font.Bold = true;
				hdrCell.Width = new Unit(100);// "100px";
				row.Cells.Add(hdrCell);

                stateSupportTable.Rows.Add(row);
            }
            catch (System.Exception ex)
            {
                ShowError(ex.Message);
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        protected override object SaveViewState()
        {
            // Save State as a cumulative array of objects.
            object baseState = base.SaveViewState();
            object[] allStates = new object[2];
            allStates[0] = baseState;
            allStates[1] = isShowStateSupport;
            return allStates;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="savedState"></param>
        protected override void LoadViewState(object savedState)
        {
            if (savedState != null)
            {
                // Load State from the array of objects that was saved at ;
                // SavedViewState.
                object[] myState = (object[])savedState;
                if (myState[0] != null)
                    base.LoadViewState(myState[0]);
                if (myState[1] != null)
                    isShowStateSupport = (bool)myState[1];
            }
        }
    }
}
