/*=====================================================================

  File:      TreeViewer.cs
  Summary:   Data Mining Web Controls is a library sample intended 
			 to extend the SQL Server Data Mining user experience 
			 to the Web. The library provides a lightweight version 
			 of the Data Mining model viewers. Using Data Mining 
			 Web Controls, complex mining models can be browsed 
			 from any machine that has Microsoft Internet Explorer 
			 installed.
  Date:      Oct 20, 2003

---------------------------------------------------------------------

  This file is part of the Microsoft SQL Server Code Samples.
  Copyright (C) 2003 Microsoft Corporation.  All rights reserved.

This source code is intended only as a supplement to Microsoft
Development Tools and/or on-line documentation.  See these other
materials for detailed information regarding Microsoft code samples.

THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
PARTICULAR PURPOSE.

======================================================= */


namespace Microsoft.AnalysisServices.DataMiningHtmlViewers
{
    using System;
    using System.CodeDom;
    using System.Collections;
    using System.ComponentModel;
    using System.ComponentModel.Design.Serialization;
    using System.Configuration;
    using System.Data;
    using System.Globalization;
    using System.IO;
    using System.Reflection;
    using System.Security.Permissions;
    using System.Web;
    using System.Web.UI;
    using System.Web.UI.WebControls;



    /// <summary>
    /// The DMDecisionTreeViewer control can be used to display 
    /// the content of models created using the 
    /// Microsoft_Decision_Trees Mining algorithm. The Decision 
    /// Trees viewer can work in two modes: Tree and 
    /// Node Details.
    /// </summary>
    [
    ToolboxData("<{0}:DMDecisionTreeViewer runat=server></{0}:DMDecisionTreeViewer>")
    ]



    [AspNetHostingPermission(SecurityAction.LinkDemand, Level = AspNetHostingPermissionLevel.Minimal)]
    [AspNetHostingPermission(SecurityAction.InheritanceDemand, Level = AspNetHostingPermissionLevel.Minimal)]
    public class DMDecisionTreeViewer : DMHtmlViewer
    {
        // Properties for creating a connection string
        private string tree;
        private string nodeId;

		// properties for rendering script links
		private string collapsedJPGPath;
		private string expandedJPGPath;
		private string scriptPath;
		private string cssPath;


        // Visual properties
        private bool showToolbar;
        private UInt32 maxVisibleHistograms;
        private UInt32 treeLineStrokeWidth;
        private TreeOverflowBehaviorType treeOverflowType;
        private TreeViewType treeViewType;
        private System.Drawing.Color treeLineStrokeColor;
        private System.Drawing.Color treeTooltipBackground;
        private System.Drawing.Color treeTooltipText;
        private TreeTooltipEvent treeTooltipEvent;
        private System.Drawing.Color treeHighDensityColor;
        private System.Drawing.Color treeLowDensityColor;
        private HistogramColorCollection historgramColors;

        // Default properties for control
        const int DefaultWidth = 400;
        const int DefaultHeight = 400;


        public DMDecisionTreeViewer()
        {
            treeLineStrokeColor = System.Drawing.Color.Black;
            treeTooltipText = System.Drawing.Color.Black;
            treeTooltipBackground = System.Drawing.Color.Beige;
            treeHighDensityColor = System.Drawing.Color.Navy;
            treeLowDensityColor = System.Drawing.Color.White;
            historgramColors = new HistogramColorCollection();

            showToolbar = true;
            tree = "";
            nodeId = "";
            treeOverflowType = TreeOverflowBehaviorType.Visible;
            treeViewType = TreeViewType.Tree;
            treeLineStrokeWidth = 1;
            treeTooltipEvent = TreeTooltipEvent.OnMouseOverEvent;

            maxVisibleHistograms = (UInt32)historgramColors.Count;

			collapsedJPGPath = string.Empty;
			expandedJPGPath = string.Empty;
			scriptPath = string.Empty;
			cssPath = string.Empty;

            this.Width = new Unit(DefaultWidth);
            this.Height = new Unit(DefaultHeight);
            
        }

		protected override void OnInit(EventArgs e)
		{
			// Register the script to be rendered as a link
			if (Page != null)
			{
				base.OnInit(e);

				collapsedJPGPath = Page.ClientScript.GetWebResourceUrl(typeof(DMDecisionTreeViewer), "col.JPG"); ;
				expandedJPGPath = Page.ClientScript.GetWebResourceUrl(typeof(DMDecisionTreeViewer), "exp.JPG"); ;
				scriptPath = Page.ClientScript.GetWebResourceUrl(typeof(DMDecisionTreeViewer), "DMDecisionTrees.js");
				cssPath = Page.ClientScript.GetWebResourceUrl(typeof(DMDecisionTreeViewer), "DMTreeViewer.css");

                string strScript =
                    "var DT_CollapsedJPGPath = \"" + collapsedJPGPath + "\";\n" +
                    "var DT_ExpandedJPGPath = \"" + expandedJPGPath + "\";\n" +
                    "var DT_CSSPath = \"" + cssPath + "\";\n" +
                    "if (document.namespaces[\"vml\"] == null)\n" +
                    "{\n" +
                    "	document.namespaces.add(\"vml\", \"urn:schemas-microsoft-com:vml\");\n" +
                    "}\n";

                

				if( !Page.ClientScript.IsClientScriptBlockRegistered("DT_Startup") )
				{
					Page.ClientScript.RegisterClientScriptBlock(this.GetType(), "DT_Startup", strScript, true);
				}

			}
		}


        /// <summary>
        /// 
        /// </summary>
        /// <value></value>
        [
        LocCategory("DataMiningContent"),
        LocDescription("DescDTTree")
        ]
        public string TargetTree
        {
            get { return tree; }
            set { tree = value; }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <value></value>
        [
        LocCategory("DataMiningContent"),
        LocDescription("DescDTNode")
        ]
        public string TargetNode
        {
            get { return nodeId; }
            set { nodeId = value; }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <value></value>
        [
        LocCategory("DataMiningLayout"),
        LocDescription("DescToolbar")
        ]
        public bool ShowToolbar
        {
            get { return showToolbar; }
            set { showToolbar = value; }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <value></value>
        [
        LocCategory("DataMiningLayout"),
        LocDescription("DescDTTarget")
        ]
        public TreeViewType ViewType
        {
            get { return treeViewType; }
            set { treeViewType = value; }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <value></value>
        [
        LocCategory("DataMiningLayout"),
        LocDescription("DescOverflow")
        ]
        public TreeOverflowBehaviorType OverflowBehavior
        {
            get { return treeOverflowType; }
            set { treeOverflowType = value; }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <value></value>
        [
        LocCategory("DataMiningLayout"),
        LocDescription("DescWidth"),
        ]
        public UInt32 LineStrokeWidth
        {
            get { return treeLineStrokeWidth; }
            set { treeLineStrokeWidth = value; }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <value></value>
        [
        LocCategory("DataMiningLayout"),
        LocDescription("DescStrokeColor")
        ]
        public System.Drawing.Color LineStrokeColor
        {
            get
            { return treeLineStrokeColor; }
            set { treeLineStrokeColor = value; }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <value></value>
        [
        LocCategory("DataMiningLayout"),
        LocDescription("DescBkColor")
        ]
        public System.Drawing.Color HistogramBackground
        {
            get { return treeTooltipBackground; }
            set { treeTooltipBackground = value; }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <value></value>
        [
        LocCategory("DataMiningLayout"),
        LocDescription("DescTextColor")
        ]
        public System.Drawing.Color HistogramText
        {
            get { return treeTooltipText; }
            set { treeTooltipText = value; }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <value></value>
        [
        LocCategory("DataMiningLayout"),
        LocDescription("DescBkHighDensity")
        ]
        public System.Drawing.Color NodeHighDensity
        {
            get { return treeHighDensityColor; }
            set { treeHighDensityColor = value; }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <value></value>
        [
        LocCategory("DataMiningLayout"),
        LocDescription("DescBkLowDensity")
        ]
        public System.Drawing.Color NodeLowDensity
        {
            get { return treeLowDensityColor; }
            set { treeLowDensityColor = value; }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <value></value>
        [
        LocCategory("DataMiningLayout"),
        LocDescription("DescHistogramEvent"),
        ]
        public TreeTooltipEvent HistogramTooltipEvent
        {
            get { return treeTooltipEvent; }
            set { treeTooltipEvent = value; }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <value></value>
        [
        System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA2227:CollectionPropertiesShouldBeReadOnly"), LocCategory("DataMiningLayout"),
        LocDescription("DescColors"),
        TypeConverter(typeof(HistogramColorCollectionConverter))
        ]
        public HistogramColorCollection HistogramColors
        {
            get { return historgramColors; }
            set { historgramColors.Copy(value); }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <value></value>
        [
        LocCategory("DataMiningLayout"),
        LocDescription("DescMaxHistograms"),
        ]
        public UInt32 MaxVisibleHistograms
        {
            get { return maxVisibleHistograms; }
            set { maxVisibleHistograms = value; }
        }

        /// <summary>
        /// Render this control to the output parameter specified.
        /// </summary>
        /// <param name="output"> The Html writer to write out to </param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1725:ParameterNamesShouldMatchBaseDeclaration")]
        protected override void Render(HtmlTextWriter output)
        {
            switch (ViewType)
            {
                case TreeViewType.Tree:
                    RenderTreeControl(output);
                    break;
                case TreeViewType.NodeDetails:
                    RenderNodeLegendControl(output);
                    break;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="output"></param>
        protected void RenderTreeControl(HtmlTextWriter output)
        {
            BuildTreeControlHtml(output);

            if (!IsDesignTime())
            {
                // Add scripting
                BuildDataScript(output);
                // Render import and loading scripts
                RenderUniqueImportScript(output);
                BuildLoadingScript(output);
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="output"></param>
        protected void RenderNodeLegendControl(HtmlTextWriter output)
        {
			if (!IsDesignTime())
			{
				// Render import and loading scripts
				RenderUniqueImportScript(output);
			}
			BuildNodeLegendControlHtml(output);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="errText"></param>
        /// <returns></returns>
        protected static string RenderDesignerError(string errText)
        {
            string retText = "<LI><SPAN><font color=\"red\">";
            retText += errText;
            retText += "</FONT></SPAN><BR></LI>";
            return retText;
        }



        // -----Rendering Code for Tree Mode-----
        #region Rendering Code for Tree Mode

        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1011:ConsiderPassingBaseTypesAsParameters"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        protected void BuildTreeControlHtml(HtmlTextWriter output)
        {
            string buffText = "";

            try
            {
                // Write the wrapper
                buffText = ("<!-- Wrapper here -->\r\n");
                buffText += "<DIV ";
                buffText += "ID='" + ID + "oWrapper' ";
                buffText += "style=\"";
                if (OverflowBehavior == TreeOverflowBehaviorType.Scroll || OverflowBehavior == TreeOverflowBehaviorType.Hidden)
                {
                    buffText += "border-style:" + BorderStyle.ToString() + "; ";
                    buffText += "border-color:" + System.Drawing.ColorTranslator.ToHtml(BorderColor) + "; ";
                    buffText += "border-width:" + BorderWidth.ToString(CultureInfo.CurrentCulture) + "; ";
                    buffText += "background-color:" + System.Drawing.ColorTranslator.ToHtml(BackColor) + "; ";
                }
                buffText += "position:absolute; ";
                buffText += "left:" + Style["LEFT"] + "; ";
                buffText += "top:" + Style["TOP"] + "; ";
                buffText += "width:" + Width.ToString(CultureInfo.CurrentCulture) + "; ";
                buffText += "height:" + Height.ToString(CultureInfo.CurrentCulture) + "; ";
                buffText += "\" ";
                buffText += ">\r\n";

                buffText += "\r\n";

                if (ShowToolbar)
                {
                    buffText += "<!-- Toolbar here -->\r\n";

                    buffText += "<input type='button' ";
                    buffText += "id='" + ID + "ZoomIn' ";
                    buffText += "value='Zoom In' ";
                    buffText += "onclick=\"DMTreeZoomIn('" + ID + "')\" ";
                    buffText += "/>\r\n";

                    buffText += "<input type='button' ";
                    buffText += "id='" + ID + "ZoomOut' ";
                    buffText += "value='Zoom Out' ";
                    buffText += "onclick=\"DMTreeZoomOut('" + ID + "')\" ";
                    buffText += "/>\r\n";


                    buffText += " &nbsp; &nbsp; Shading: ";
                    buffText += "<select ";
                    buffText += "name='" + ID + "_attrValues' ";
                    buffText += "id='" + ID + "_attrValues' ";
                    buffText += "onchange=\"DMTreeDensityFactorChanged('" + ID + "')\" ";
                    buffText += "></select>\r\n";

                    buffText += "<hr width=100%>\r\n";
                    output.WriteLine("<!-- End Toolbar-->\r\n");
                }

                buffText += "\r\n";

                // Write the container
                buffText += ("<!-- Container here -->\r\n");
                buffText += "<DIV ";
                buffText += "ID='" + ID + "oContainer' ";
                buffText += "style=\"";
                if (OverflowBehavior == TreeOverflowBehaviorType.Scroll || OverflowBehavior == TreeOverflowBehaviorType.Hidden)
                {
                    buffText += "background-color:" + System.Drawing.ColorTranslator.ToHtml(BackColor) + "; ";
                }
                buffText += "position:absolute; ";
                buffText += "left:1px;";

                buffText += "width:" + (Width.Value - 2 * BorderWidth.Value) + "; ";

                double dRealHeight = Height.Value;
                dRealHeight -= 2 * BorderWidth.Value;
                if (ShowToolbar)
                {
                    // Leave room for the toolbar
                    buffText += "top:60px;";
                    dRealHeight -= 60;
                }
                else
                {
                    buffText += "top:" + BorderWidth.Value.ToString(CultureInfo.CurrentCulture) + ";";
                }

                buffText += "height:" + dRealHeight.ToString(CultureInfo.CurrentCulture) + "; ";
                buffText += "\" ";
                buffText += ">\r\n";
                buffText += "\r\n";

                if (IsDesignTime())
                {
                    buffText += "<SPAN style='align=center valign=center'><CENTER><H1>DataMining Html Tree Viewer</H1></CENTER></SPAN>";
                    buffText += "<SPAN style='align=center valign=center'><CENTER><H3>Mode: Tree</H3></CENTER></SPAN>";
                    buffText += "<UL>";

                    if (TargetTree.Length == 0)
                    {
                        buffText += RenderDesignerError("'Tree' is Empty -- the control will not work unless the Tree property is assigned to");
                    }

                    if (Model.Length == 0)
                    {
                        buffText += RenderDesignerError("'Model' is Empty -- the control will not work unless the Model property is assigned to");
                    }

                    if (MaxVisibleHistograms > HistogramColors.Count)
                    {
                        buffText += RenderDesignerError("'HistogramColors' should contain at least 'MaxVisibleHistograms' colors");
                    }

                    if (MaxVisibleHistograms < 2)
                    {
                        buffText += RenderDesignerError("MaxVisibleHistograms will be automatically changed to be at least 2.");
                    }

                    buffText += "</UL>";
                    buffText += "<br>";
                }

                // Close the Container
                buffText += "</DIV>\r\n";

                // Close the Wrapper
                buffText += "</DIV>\r\n";
            }
            catch (System.Exception ex)
            {
                buffText = "<H1><FONT color='red'>Error: " + ex.Message + "</font></H1>\r\n";
            }

            output.Write(buffText);
        }

        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1011:ConsiderPassingBaseTypesAsParameters")]
        protected void RenderUniqueImportScript(HtmlTextWriter output)
        {
            /*string strScriptSrc = GetClientFileUrl("DMDecisionTrees.js");*/
            string strImportScript = "\r\n<script language=\"javascript\" " +
                                     " src=\"" + scriptPath + "\"></script>\r\n";

            output.Write(strImportScript);
			//if (!Page.ClientScript.IsClientScriptIncludeRegistered("DT_script"))
				// Page.ClientScript.RegisterClientScriptInclude("DT_script", scriptPath);

			
			//if (LinkedStyleSheets.Count > 0) 

        }

        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1011:ConsiderPassingBaseTypesAsParameters")]
        protected void BuildLoadingScript(HtmlTextWriter output)
        {
            string buffText = "";

            // Write the wrapper
            buffText = ("<!-- Loader for the " + ID + " decision tree -->\r\n");
            buffText += "\r\n<script language=\"javascript\">  \r\n";
            buffText += "{\r\n";
            buffText += "//////////////////////////////////////////////\r\n";
            buffText += "// Init the TreeID tree control\r\n";
            buffText += "//////////////////////////////////////////////\r\n";
            buffText += "// Build the nodes\r\n";
            buffText += "\tvar " + ID + "arNodes = DMTreeBuildNodes(" + ID + "NodeData);\r\n";
            buffText += "\t\r\n";
            buffText += "\t// Init Tree\r\n";
            buffText += "\tDMTreeInitialize(\"" + ID + "\");\r\n";
            buffText += "//////////////////////////////////////////////\r\n";
            buffText += "// END TreeID tree control\r\n";
            buffText += "//////////////////////////////////////////////\r\n";
            buffText += "}\r\n";
            buffText += "</script>\r\n";

            output.Write(buffText);
        }

        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1011:ConsiderPassingBaseTypesAsParameters"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        protected void BuildDataScript(HtmlTextWriter output)
        {
            string buffText = "";

            try
            {
                TreeViewerModel model = new TreeViewerModel();

                model.TotalHistograms = this.MaxVisibleHistograms;
                if (model.TotalHistograms < 2)
                    model.TotalHistograms = 2;
                model.DatabaseConnection = this.Connection;
                model.HighDensityColor = this.NodeHighDensity;
                model.LowDensityColor = this.NodeLowDensity;
                model.HistogramColorsCollection = this.HistogramColors;
                model.ServerName = this.Server;
                model.CatalogName = this.Database;
                model.ModelName = this.Model;
                model.TreeName = this.TargetTree;


                // Write beginning of the script
                buffText += "\r\n<!--Start Data Scripting for " + ID + "-->\r\n";
                buffText += "<script id='" + ID + "DataHere' language=javascript>\r\n";
                buffText += "var " + ID + "TooltipBackground= \"" + System.Drawing.ColorTranslator.ToHtml(HistogramBackground) + "\";\r\n";
                buffText += "var " + ID + "TooltipTextColor= \"" + System.Drawing.ColorTranslator.ToHtml(HistogramText) + "\";\r\n";
                buffText += "var " + ID + "TooltipEvent= \"" + this.HistogramTooltipEvent + "\";\r\n";
                buffText += "var " + ID + "LineStrokeWeight= \"" + this.LineStrokeWidth + "\";\r\n";
                buffText += "var " + ID + "LineStrokeColor= \"" + System.Drawing.ColorTranslator.ToHtml(LineStrokeColor) + "\";\r\n";
                buffText += "var " + ID + "ScrollBars= \"" + OverflowBehavior + "\";\r\n";
                buffText += "var " + ID + "HighDensity_R= " + this.NodeHighDensity.R.ToString(CultureInfo.CurrentCulture) + ";\r\n";
                buffText += "var " + ID + "HighDensity_G= " + this.NodeHighDensity.G.ToString(CultureInfo.CurrentCulture) + ";\r\n";
                buffText += "var " + ID + "HighDensity_B= " + this.NodeHighDensity.B.ToString(CultureInfo.CurrentCulture) + ";\r\n";
                buffText += "var " + ID + "LowDensity_R= " + this.NodeLowDensity.R.ToString(CultureInfo.CurrentCulture) + ";\r\n";
                buffText += "var " + ID + "LowDensity_G= " + this.NodeLowDensity.G.ToString(CultureInfo.CurrentCulture) + ";\r\n";
                buffText += "var " + ID + "LowDensity_B= " + this.NodeLowDensity.B.ToString(CultureInfo.CurrentCulture) + ";\r\n";

                // Write data
                //

                System.Collections.ArrayList arNodes = new System.Collections.ArrayList();
                model.GetContent(ref arNodes);

                buffText += WriteHistogramInfo(ref model, ref arNodes);

                buffText += WriteNodeData(ref arNodes);

                // Write end of script
                buffText += "</script>\r\n";
                buffText += "<!--End Data Scripting for " + ID + "-->\r\n\r\n\r\n";
            }
            catch (System.Exception ex)
            {
                buffText = "Error: " + ex.Message + "\r\n";
            }

            output.Write(buffText);
        }

        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1045:DoNotPassTypesByReference"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1045:DoNotPassTypesByReference"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        protected string WriteHistogramInfo(ref TreeViewerModel model, ref System.Collections.ArrayList arNodes)
        {
            string buffText = "";

            try
            {
                int i = 0;
                int histCount = 0;
                if (arNodes.Count > 0)
                {
                    TreeViewerNode node = (TreeViewerNode)arNodes[0];
                    histCount = node.DistributionsList.Count;
                }

                buffText += "var " + ID + "cHistograms = " + histCount.ToString(CultureInfo.CurrentCulture) + "\r\n";

                System.Collections.ArrayList arHisto = model.GetHistogramsInfo();
                buffText += "var " + ID + "arHistogramInfo = new Array (\r\n";

                for (i = 0; i < histCount; i++)
                {
                    HistogramInfo hInfo = (HistogramInfo)arHisto[i];
                    buffText += "\"" + hInfo.Name + "\"";

                    if (i < histCount - 1)
                    {
                        buffText += ",\r\n";
                    }
                    else
                    {
                        // Last
                        buffText += ");\r\n";
                    }
                }

                buffText += "var " + ID + "arHistogramColors = new Array (\r\n";
                for (i = 0; i < histCount; i++)
                {
                    HistogramInfo hInfo = (HistogramInfo)arHisto[i];
                    buffText += "\"" + hInfo.Color + "\"";
                    if (i < histCount - 1)
                    {
                        buffText += ",\r\n";
                    }
                    else
                    {
                        // Last
                        buffText += ");\r\n";
                    }
                }

                buffText += "\r\n";
                buffText += "var " + ID + "arHistograms = new Array (\r\n";
                for (i = 0; i < arNodes.Count; i++)
                {
                    TreeViewerNode node = (TreeViewerNode)arNodes[i];

                    if (histCount != node.DistributionsList.Count)
                    {
                        //throw new System.Exception("Number of distributions is not constant");
                    }

                    buffText += "                  ";
                    for (int j = 0; j < histCount; j++)
                    {
                        Dist curDist = (Dist)node.DistributionsList[j];
                        int iNumber = (curDist != null) ? curDist.histogram : 0;
                        buffText += iNumber.ToString(CultureInfo.CurrentCulture);
                        if ((i < arNodes.Count - 1) || (j < histCount - 1))
                        {
                            buffText += ", ";
                        }
                    }
                    buffText += "\r\n";
                }
                buffText += ");\r\n";
            }
            catch (System.Exception ex)
            {
                buffText = "Error: " + ex.Message + "\r\n";
            }

            return buffText;
        }


        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1045:DoNotPassTypesByReference"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        protected string WriteNodeData(ref System.Collections.ArrayList arNodes)
        {
            string retText = "";

            try
            {
                int i = 0;
                int histCount = 0;

                if (arNodes.Count > 0)
                {
                    TreeViewerNode node = (TreeViewerNode)arNodes[0];
                    histCount = node.DistributionsList.Count;
                }

                retText += "\r\n";
                retText += "var " + ID + "arProbabilities = new Array (\r\n";
                for (i = 0; i < arNodes.Count; i++)
                {
                    TreeViewerNode node = (TreeViewerNode)arNodes[i];

                    if (histCount != node.DistributionsList.Count)
                    {
                        //throw new System.Exception("Number of distributions is not constant");
                    }

                    retText += "                  ";
                    for (int j = 0; j < node.DistributionsList.Count; j++)
                    {
                        Dist curDist = (Dist)node.DistributionsList[j];
                        double dProb = (curDist != null) ? 100 * curDist.probability : 0.0;

                        retText += "\"" + dProb.ToString("F", CultureInfo.CurrentCulture) + "%\"";
                        if ((i < arNodes.Count - 1) || (j < histCount - 1))
                        {
                            retText += ", ";
                        }
                    }
                    retText += "\r\n";
                }
                retText += ");\r\n";

                retText += "\r\n";
                retText += "var " + ID + "arCasesSupport = new Array (\r\n";
                for (i = 0; i < arNodes.Count; i++)
                {
                    TreeViewerNode node = (TreeViewerNode)arNodes[i];

                    if (histCount != node.DistributionsList.Count)
                    {
                        //throw new System.Exception("Number of distributions is not constant");
                    }

                    retText += "                  ";
                    for (int j = 0; j < node.DistributionsList.Count; j++)
                    {
                        Dist curDist = (Dist)node.DistributionsList[j];
                        double dSupport = (curDist != null) ? curDist.support : 0.0;
                        retText += dSupport.ToString(CultureInfo.CurrentCulture);
                        if ((i < arNodes.Count - 1) || (j < histCount - 1))
                        {
                            retText += ", ";
                        }
                    }
                    retText += "\r\n";
                }
                retText += ");\r\n";

                retText += "\r\n";
                retText += "var " + ID + "NodeData = new Array (\r\n";

                for (i = 0; i < arNodes.Count; i++)
                {
                    TreeViewerNode node = (TreeViewerNode)arNodes[i];
                    retText += "\"" + node.Caption + "\", ";
                    if (node.ParentId >= 0)
                    {
                        retText += node.ParentId.ToString(CultureInfo.CurrentCulture) + ", ";
                    }
                    else
                    {
                        retText += "null, ";
                    }
                    retText += System.Convert.ToInt32(node.Support) + ", ";
                    retText += "\"" + node.BackgroundColor + "\", ";
                    retText += "\"" + node.TextColor + "\"";

                    if (i < arNodes.Count - 1)
                    {
                        retText += ", \r\n";
                    }
                    else
                    {
                        retText += "); \r\n";
                    }
                }
            }
            catch (System.Exception ex)
            {
                retText = "Error: " + ex.Message + "\r\n";
            }

            return retText;
        }

        #endregion



        // -----Rendering Code for Node Legend Mode-----
        #region Rendering Code for Node Legend Mode
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        protected void BuildNodeLegendControlHtml(HtmlTextWriter output)
        {
            string buffText = "";

            try
            {
                // Write the wrapper
                buffText = ("<!-- Wrapper here -->\r\n");
                buffText += "<DIV ";
                buffText += "ID='" + ID + "oWrapper' ";
                buffText += "style=\"";
                if (OverflowBehavior == TreeOverflowBehaviorType.Scroll || OverflowBehavior == TreeOverflowBehaviorType.Hidden)
                {
                    buffText += "border-style:" + BorderStyle.ToString() + "; ";
                    buffText += "border-color:" + System.Drawing.ColorTranslator.ToHtml(BorderColor) + "; ";
                    buffText += "border-width:" + BorderWidth.ToString(CultureInfo.CurrentCulture) + "; ";
                    buffText += "background-color:" + System.Drawing.ColorTranslator.ToHtml(BackColor) + "; ";
                }
                buffText += "position:absolute; ";
                buffText += "left:" + Style["LEFT"] + "; ";
                buffText += "top:" + Style["TOP"] + "; ";
                buffText += "width:" + Width.ToString(CultureInfo.CurrentCulture) + "; ";
                buffText += "height:" + Height.ToString(CultureInfo.CurrentCulture) + "; ";
                buffText += "\" ";
                buffText += ">\r\n";

                buffText += "\r\n";
            }
            catch (System.Exception ex)
            {
                buffText = "<H1><FONT color='red'>Error: " + ex.Message + "</font></H1>\r\n";
            }
            output.Write(buffText);


            if (IsDesignTime())
            {
                buffText = "";

                try
                {
                    buffText += "<SPAN style='align=center valign=center'><CENTER><H1>DataMining Html Tree Viewer</H1></CENTER></SPAN>";
                    buffText += "<SPAN style='align=center valign=center'><CENTER><H3>Mode: Node Details</H3></CENTER></SPAN>";
                    buffText += "<UL>";

                    if (TargetNode.Length == 0)
                    {
                        buffText += RenderDesignerError("'NodeId' is Empty -- the control will not work unless the NodeId property is assigned to");
                    }

                    if (Model.Length == 0)
                    {
                        buffText += RenderDesignerError("'Model' is Empty -- the control will not work unless the Model property is assigned to");
                    }

                    if (MaxVisibleHistograms > HistogramColors.Count)
                    {
                        buffText += RenderDesignerError("'HistogramColors' should contain at least 'MaxVisibleHistograms' colors");
                    }

                    if (MaxVisibleHistograms < 2)
                    {
                        buffText += RenderDesignerError("MaxVisibleHistograms will be automatically changed to be at least 2.");
                    }

                    buffText += "</UL>";
                    buffText += "<br>";
                }
                catch (System.Exception ex)
                {
                    buffText = "<H1><FONT color='red'>Error: " + ex.Message + "</font></H1>\r\n";
                }
                output.Write(buffText);
            }
            else
            {
                // Add Html
                BuildNodeDetailHtml(output);
            }

            // Close the Wrapper
            output.Write("</DIV>\r\n");
        }


        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1011:ConsiderPassingBaseTypesAsParameters"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        protected void BuildNodeDetailHtml(HtmlTextWriter output)
        {
            string buffText = "";

            try
            {
                TreeViewerModel model = new TreeViewerModel();

                model.TotalHistograms = this.MaxVisibleHistograms;
                if (model.TotalHistograms < 2)
                    model.TotalHistograms = 2;
                model.DatabaseConnection = this.Connection;
                model.HighDensityColor = this.NodeHighDensity;
                model.LowDensityColor = this.NodeLowDensity;
                model.HistogramColorsCollection = this.HistogramColors;
                model.ServerName = this.Server;
                model.CatalogName = this.Database;
                model.ModelName = this.Model;
                model.TreeName = this.TargetTree;

                // Write data
                //
                System.Collections.ArrayList arNodes = new System.Collections.ArrayList();
                model.GetSingleNode(TargetNode, ref arNodes);

                if (arNodes.Count != 1)
                    return;

                TreeViewerNode node = (TreeViewerNode)arNodes[0];

                // Now write the Html
                //
                buffText += "<CENTER><H3>" + node.Caption + "</H3></CENTER>\r\n";
                buffText += "<SPAN>&nbsp;" + Properties.Resources.StrTotalCases + System.Convert.ToInt32(node.Support) + "</SPAN></BR>\r\n";

                if (node.IsRegression)
                {
                    buffText += BuildRegressionHeader(ref node);
                }
                else
                {
                    buffText += BuildDiscreteHeader(ref node);
                }

                buffText += "<br>";

                // Now write the tables
                //
                if (node.IsRegression)
                {
                    buffText += BuildRegressionTable(ref node);
                }
                else
                {
                    buffText += BuildDiscreteTable(ref node);
                }
            }
            catch (System.Exception ex)
            {
                buffText = "<H1><FONT color='red'>Error: " + ex.Message + "</font></H1>\r\n";
            }

            output.Write(buffText);
        }

        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1045:DoNotPassTypesByReference")]
        public static string BuildDiscreteHeader(ref TreeViewerNode node)
        {
            string buffText = "";
            buffText += "<SPAN>&nbsp; " + node.AttributeName + "</SPAN><br>\r\n";
            return buffText;
        }

        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1045:DoNotPassTypesByReference"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        public static string BuildRegressionHeader(ref TreeViewerNode node)
        {
            string buffText = "";

            try
            {
                double dMissing = 0.0;
                string regressionFormula = "[" + node.AttributeName + "] = ";

                if (node.TermsList.Count == 0)
                {
                    node.BuildTerms();
                }

                if (node.TermsList.Count > 0)
                {
                    Term freeTerm = (Term)node.TermsList[0];

                    double dFreeTermCoeff = Math.Round(freeTerm.coefficient, 3);

                    regressionFormula += dFreeTermCoeff.ToString(CultureInfo.CurrentCulture);
                    for (int index = 1; index < node.TermsList.Count; index++)
                    {
                        Term term = (Term)node.TermsList[index];

                        double dCoeff = (double)term.coefficient;
                        dCoeff = Math.Round(dCoeff, 3);

                        regressionFormula += " ";
                        if (dCoeff > 0)
                            regressionFormula += "+";

                        // otherwise, '-' comes from ToString
                        regressionFormula += dCoeff.ToString(CultureInfo.CurrentCulture) + "*[" + term.nameText + "]";
                    }

                    double stdDev = Math.Round(freeTerm.stdDev, 3);
                    if (stdDev != 0)
                    {
                        regressionFormula += "  " + stdDev.ToString(CultureInfo.CurrentCulture);
                    }
                }

                buffText += "<SPAN>&nbsp; Missing: " + System.Convert.ToInt32(dMissing, CultureInfo.CurrentCulture).ToString(CultureInfo.CurrentCulture) + "</SPAN><br>\r\n";
                buffText += "<SPAN>&nbsp; " + regressionFormula + "</SPAN><br>\r\n";
                buffText += "<xml:namespace ns=\"urn:schemas-microsoft-com:vml\" prefix=\"vml\"/>\r\n";
            }
            catch (System.Exception ex)
            {
                buffText = "<H1><FONT color='red'>Error: " + ex.Message + "</font></H1>\r\n";
            }

            return buffText;
        }


        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1045:DoNotPassTypesByReference"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        public static string BuildRegressionTable(ref TreeViewerNode node)
        {
            string retText = "";

            try
            {
                retText += "<TABLE align='center' width=95% border=1><TR align = 'center'>";

                string prefixText = "<TD>";
                string suffixText = "</TD>";

                // Add table columns
                retText += prefixText + "Term" + suffixText;
                retText += prefixText + " &nbsp; " + suffixText;
                retText += prefixText + "Coefficient" + suffixText;
                // Histogram column has fixed width
                retText += "<TD width='100' colspan=2><C>Histogram</C>" + suffixText;
                retText += "</TR>";

                for (int i = 0; i < node.TermsList.Count; i++)
                {
                    Term term = (Term)node.TermsList[i];

                    // Generate the histogram data -- attr value, support, probability
                    retText += "<TR>";
                    prefixText = "<TD>";
                    suffixText = "</TD>\r\n";

                    string name = term.nameText;
                    if (name.Length == 0)
                        name = "&nbsp;";
                    retText += prefixText + name + suffixText;
                    retText += prefixText + "*" + suffixText;
                    double dCoeff = Math.Round(term.coefficient, 3);

                    retText += prefixText + dCoeff.ToString(CultureInfo.CurrentCulture) + suffixText;
                    //retText += "\r\n<TD width=110>";

                    // Generate the histogram bar
                    if (i == 0)
                    {
                        retText += RenderFreeCoefficientGraph(ref node, ref term);
                    }
                    else
                    {
                        retText += RenderPolyCoefficientGraph(ref node, ref term);
                    }
                    //retText += "</TD>\r\n";
                    retText += "</TR>\r\n";
                }

                retText += "</TABLE>";
            }
            catch (System.Exception ex)
            {
                retText = "<H1><FONT color='red'>Error: " + ex.Message + "</font></H1>\r\n";
            }

            return retText;
        }

        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        string BuildDiscreteTable(ref TreeViewerNode node)
        {
            string retText = "";

            try
            {
                retText += "<TABLE align='center' width=95% border=1><TR align = 'center'>";

                string prefixText = "<TD>";
                string suffixText = "</TD>";

                // Add table columns
                retText += prefixText + Properties.Resources.StrValue + suffixText;
                retText += prefixText + Properties.Resources.StrCases + suffixText;
                retText += prefixText + Properties.Resources.StrProbability + suffixText;
                // Histogram column has fixed width
                retText += "<TD width='110'>" + Properties.Resources.StrHistogram + suffixText;
                retText += "</TR>\r\n";

                for (int i = 0; i < node.DistributionsList.Count; i++)
                {
                    Dist dist = (Dist)node.DistributionsList[i];

                    // Generate the histogram data -- attr value, support, probability
                    retText += "<TR>";
                    prefixText = "<TD>";
                    suffixText = "</TD>\r\n";
                    retText += prefixText + dist.name + suffixText;
                    retText += prefixText + System.Convert.ToInt32(dist.support, CultureInfo.CurrentCulture).ToString(CultureInfo.CurrentCulture) + suffixText;
                    double dProbability = Math.Round(dist.probability, 3);
                    retText += prefixText + dProbability.ToString(CultureInfo.CurrentCulture) + suffixText;

                    retText += "<TD >";

                    int width = System.Convert.ToInt32(100 * dProbability);
                    // Generate the histogram bar
                    retText += "<DIV STYLE=\"font-size:0%; border:none; height:10px;left:10px;";
                    retText += "width:" + width.ToString(CultureInfo.CurrentCulture) + "px;";
                    retText += "background-color:" + System.Drawing.ColorTranslator.ToHtml(historgramColors[i]) + ";";
                    retText += "\"></DIV>";
                    retText += "</TD>";
                    retText += "</TR>\r\n";
                }

                retText += "</TABLE>";
            }
            catch (System.Exception ex)
            {
                retText = "<H1><FONT color='red'>Error: " + ex.Message + "</font></H1>\r\n";
            }

            return retText;
        }

        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1045:DoNotPassTypesByReference"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1045:DoNotPassTypesByReference"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        public static string RenderFreeCoefficientGraph(ref TreeViewerNode node, ref Term term)
        {
            string buffText = "";

            try
            {
                int scaleWidth = 100;
                int val = scaleWidth / 2, meanVal = scaleWidth / 2;

                if (node.ValueRangeMax > node.ValueRangeMin)
                {
                    double dVal = (term.coefficient - node.ValueRangeMin) / (node.ValueRangeMax - node.ValueRangeMin);
                    dVal = dVal * scaleWidth;
                    meanVal = System.Convert.ToInt32(dVal);


                    dVal = (term.stdDev - node.ValueRangeMin) / (node.ValueRangeMax - node.ValueRangeMin);
                    dVal = dVal * scaleWidth;
                    val = System.Convert.ToInt32(dVal);

					if (System.Math.Abs(val) > System.Math.Abs(meanVal))
                    {
                        val = meanVal;
                    }

					if (System.Math.Abs(val) > (scaleWidth - meanVal))
                    {
                        val = scaleWidth - meanVal;
                    }
                }

                
                if (term.coefficient > 0)
				{
					// Bar to the right 

					buffText += "\r\n<TD>&nbsp;</TD><TD width=110 align=\"left\"><DIV STYLE=\"align=left; font-size:0%; border:none; height:10px;left:5px;";
					buffText += "width:100px;";
					buffText += "background-color:Navy";
					buffText += "\"></DIV></TD>";

				}
				else
				{
					// bar to the left
					buffText += "\r\n<TD width=110 align=\"right\"><DIV STYLE=\"align=right; font-size:0%; border:none; height:10px;left:5px;";
					buffText += "width:100px;";
					buffText += "background-color:Navy";
					buffText += "\"></DIV></TD><TD>&nbsp;</TD>";
				}

            }
            catch (System.Exception ex)
            {
                buffText = "<H1><FONT color='red'>Error: " + ex.Message + "</font></H1>\r\n";
            }

            return buffText;
        }

        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1045:DoNotPassTypesByReference"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1045:DoNotPassTypesByReference"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1031:DoNotCatchGeneralExceptionTypes")]
        public static string RenderPolyCoefficientGraph(ref TreeViewerNode node, ref Term term)
        {
            string buffText = "";

            try
            {
                int scaleWidth = 100;

                int val;
                val = 0 - 10;

                // Compute val here

                if (node.ValueRangeMax <= node.ValueRangeMin)
                    return "";

                double dAbsoluteCoefficient = Math.Abs(term.coefficient);

                double dVal = 0.0;
                if (term.stdDev != 0)
                {
                    double dRelativeCoefficient = dAbsoluteCoefficient * term.stdDev / (node.ValueRangeMax - node.ValueRangeMin);
                    dVal = dRelativeCoefficient * scaleWidth;
                }
                else
                {
                    //Same hack as in bienv
                    double dblCeilHack = Math.Ceiling(Math.Log10(dAbsoluteCoefficient));
                    double dblScaleHack = Math.Pow(10.0, dblCeilHack);
                    dVal = (0.25 * dAbsoluteCoefficient / dblScaleHack * scaleWidth); 
                }

                if (term.coefficient > 0)
                {
                    val = System.Convert.ToInt32(dVal);
                }
                else
                {
                    val = 0 - System.Convert.ToInt32(dVal, CultureInfo.CurrentCulture);
                }


				if( val > 0) 
				{
					// Bar to the right 

					buffText += "\r\n<TD>&nbsp;</TD><TD width=110 align=\"left\"><DIV STYLE=\"align=left; font-size:0%; border:none; height:10px;left:5px;";
					buffText += "width:" + val.ToString(CultureInfo.InvariantCulture) + "px;";
					buffText += "background-color:Navy";
					buffText += "\"></DIV></TD>";

				}
				else
				{
					// bar to the left
					buffText += "\r\n<TD width=110 align=\"right\"><DIV STYLE=\"align=right; font-size:0%; border:none; height:10px;left:5px;";
					buffText += "width:" + (-val).ToString(CultureInfo.InvariantCulture) + "px;";
					buffText += "background-color:Navy";
					buffText += "\"></DIV></TD><TD>&nbsp;</TD>";
				}

				
            }
            catch (System.Exception ex)
            {
                buffText = "<H1><FONT color='red'>Error: " + ex.Message + "</font></H1>\r\n";
            }
            return buffText;
        }

        #endregion
    }


}
