/*=====================================================================

  File:      TreeViewerHelpers.cs
  Summary:   Data Mining Web Controls is a library sample intended 
			 to extend the SQL Server Data Mining user experience 
			 to the Web. The library provides a lightweight version 
			 of the Data Mining model viewers. Using Data Mining 
			 Web Controls, complex mining models can be browsed 
			 from any machine that has Microsoft Internet Explorer 
			 installed.
  Date:      Oct 20, 2003

---------------------------------------------------------------------

  This file is part of the Microsoft SQL Server Code Samples.
  Copyright (C) 2003 Microsoft Corporation.  All rights reserved.

This source code is intended only as a supplement to Microsoft
Development Tools and/or on-line documentation.  See these other
materials for detailed information regarding Microsoft code samples.

THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
PARTICULAR PURPOSE.

======================================================= */
namespace Microsoft.AnalysisServices.DataMiningHtmlViewers
{
    using System;
    using System.Web.UI;
    using System.Web.UI.WebControls;
    using System.ComponentModel;
    using System.Data;
    using System.Collections;
    using System.Configuration;
    using System.Reflection;
    using System.IO;
    using System.ComponentModel.Design.Serialization;
    using System.CodeDom;
    using System.Runtime.Serialization;
    using System.Globalization;
    using System.Xml.Serialization;


    /// <summary>
    /// 
    /// </summary>
    public class HistogramColorCollectionConverter : TypeConverter
    {
        /// <summary>
        /// 
        /// </summary>
        /// <param name="context"></param>
        /// <param name="destinationType"></param>
        /// <returns></returns>
        public override bool CanConvertTo(
            ITypeDescriptorContext context,
            Type destinationType)
        {
            if (destinationType == typeof(string))
                return true;
            return base.CanConvertTo(context, destinationType);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="context"></param>
        /// <param name="destinationType"></param>
        /// <returns></returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Naming", "CA1725:ParameterNamesShouldMatchBaseDeclaration")]
        public override bool CanConvertFrom(
            ITypeDescriptorContext context,
            Type destinationType)
        {
            if (destinationType == typeof(string))
                return true;
            return base.CanConvertTo(context, destinationType);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="context"></param>
        /// <param name="culture"></param>
        /// <param name="value"></param>
        /// <param name="destinationType"></param>
        /// <returns></returns>
        public override object ConvertTo(
            ITypeDescriptorContext context,
            CultureInfo culture,
            object value,
            Type destinationType)
        {
            // If the destination type is a string, return col.ToString()
            if (destinationType == typeof(string))
            {
                HistogramColorCollection col = (HistogramColorCollection)value;
                return col.ToString();
            }
            return base.ConvertTo(context, culture, value, destinationType);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="context"></param>
        /// <param name="culture"></param>
        /// <param name="value"></param>
        /// <returns></returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1800:DoNotCastUnnecessarily")]
        public override object ConvertFrom(
            ITypeDescriptorContext context,
            CultureInfo culture,
            object value)
        {
            // If the original type is a string, use the protected string ctor
            if (value is string || value == null)
            {
                string colDesc = (string)value;
                HistogramColorCollection col = new HistogramColorCollection();
                col.FromString(colDesc);

                return col;
            }

            return base.ConvertFrom(context, culture, value);
        }
    }

    /// <summary>
    /// 
    /// </summary>
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1010:CollectionsShouldImplementGenericInterface"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1039:ListsAreStronglyTyped"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1035:ICollectionImplementationsHaveStronglyTypedMembers"), TypeConverter(typeof(HistogramColorCollectionConverter))]
    [Serializable()]
    public class HistogramColorCollection : CollectionBase
    {
        public HistogramColorCollection()
        {
            // Providing 6 colors, by default
            this[0] = System.Drawing.Color.Lime;
            this[1] = System.Drawing.Color.Magenta;
            this[2] = System.Drawing.Color.Red;
            this[3] = System.Drawing.Color.Yellow;
            this[4] = System.Drawing.Color.Blue;
            this[5] = System.Drawing.Color.Crimson;
        }

        /// <summary>
        /// Item by index
        /// </summary>
        public System.Drawing.Color this[int index]   // indexer declaration
        {
            get
            {
                return (System.Drawing.Color)InnerList[index];
            }
            set
            {
                while (InnerList.Count <= index)
                {
                    InnerList.Add(System.Drawing.Color.FromName("0"));
                }
                InnerList[index] = value;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="col"></param>
        public void Copy(HistogramColorCollection col)
        {
            InnerList.Clear();
            InnerList.InsertRange(0, col);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            string retText = "";
            for (int i = 0; i < InnerList.Count; i++)
            {
                if (i > 0)
                    retText += ",";
                retText += this[i].Name;
            }
            return retText;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="valueText"></param>
        public void FromString(string valueText)
        {
            InnerList.Clear();

            if (valueText == null)
                return;

            string[] arColors = valueText.Split(new char[] { ',' });
            for (int i = 0; i < arColors.Length; i++)
            {
                InnerList.Add(System.Drawing.Color.FromName(arColors[i]));
            }
        }
    }


    /// <summary>
    /// 
    /// </summary>
    public class Dist
    {
        internal int histogram;
        internal string attributeName;
        internal string name;
        internal double support;
        internal double probability;
        internal double variance;
        internal object attributeValue;
		internal int valueType;
    }


    /// <summary>
    /// 
    /// </summary>
    class DistComparer : IComparer
    {
        public virtual int Compare(object x, object y)
        {
            Dist dX = (Dist)x;
            Dist dY = (Dist)y;
            return dX.support.CompareTo(dY.support);
        }
    };


    /// <summary>
    /// 
    /// </summary>
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1815:OverrideEqualsAndOperatorEqualsOnValueTypes")]
    public struct Term
    {
        internal string nameText;
        internal double coefficient;
        internal double stdDev;
    };


    /// <summary>
    /// 
    /// </summary>
    public class TreeViewerNode
    {
        private int parentID;
        private bool isRegression;
        private string attributeName;
        private string caption;
        private string description;
        private string backgroundColor;
        private string textColor;
        private double support;
        private double valueRangeMin;
        private double valueRangeMax;
        private double regressorMissing;
        private ArrayList termsArrayList;
        private ArrayList distributionsArrayList;

        public TreeViewerNode()
        {
            this.termsArrayList = new ArrayList();
            this.distributionsArrayList = new ArrayList();
            this.parentID = -1;
        }

        public int ParentId
        {
            get { return this.parentID; }
            set { this.parentID = value; }
        }

        public bool IsRegression
        {
            get { return this.isRegression; }
            set { this.isRegression = value; }
        }
        
        public string AttributeName
        {
            get { return this.attributeName; }
            set { this.attributeName = value; }
        }
        
        public string Caption
        {
            get { return this.caption; }
            set { this.caption = value; }
        }
        
        public string Description
        {
            get { return this.description; }
            set { this.description = value; }
        }
        
        public string BackgroundColor
        {
            get { return this.backgroundColor; }
            set { this.backgroundColor = value; }
        }
        
        public string TextColor
        {
            get { return this.textColor; }
            set { this.textColor = value; }
        }
        
        public double Support
        {
            get { return this.support; }
            set { this.support = value; }
        }
        
        public double ValueRangeMin
        {
            get { return this.valueRangeMin; }
            set { this.valueRangeMin = value; }
        }
        
        public double ValueRangeMax
        {
            get { return this.valueRangeMax; }
            set { this.valueRangeMax = value; }
        }
        
        public ArrayList TermsList
        {
            get { return this.termsArrayList; }
        }
        
        public ArrayList DistributionsList
        {
            get { return this.distributionsArrayList; }
        }

        /// <summary>
        /// 
        /// </summary>
        public void BuildTerms()
        {
            for (int index = 0; index < distributionsArrayList.Count; index++)
            {
                Dist dist = (Dist)distributionsArrayList[index];

                if (dist.attributeName == attributeName)
                {
                    if (dist.name == "Missing")
                        regressorMissing = dist.support;
                }
                else
                {
                    if (dist.valueType == 7 || dist.valueType == 3 || dist.valueType == 11)
					{
						// Only add if coefficient(7) or intercept(11)  -- free term
						Term term = new Term();
						term.nameText = dist.attributeName;
						term.coefficient = (double)dist.attributeValue;
						term.stdDev = Math.Sqrt(dist.variance);
						if (term.nameText.Length == 0)
						{
							// The 'free' coefficient should be the first
							termsArrayList.Insert(0, term);
						}
						else
						{
							termsArrayList.Add(term);
						}
					}
                }
            }
        }
    };


    /// <summary>
    /// 
    /// </summary>
    public class HistogramInfo
    {
        private int index;
        private string color;
        private string name;

        public int Index
        {
            get { return index; }
            set { index = value; }
        }

        public string Color
        {
            get { return color; }
            set { color = value; }
        }

        public string Name
        {
            get { return name; }
            set { name = value; }
        }
    };


    /// <summary>
    /// 
    /// </summary>
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1001:TypesThatOwnDisposableFieldsShouldBeDisposable")]
    public class TreeViewerModel
    {
        private double _maxSupport;
        private bool _useCatchAllState;
        private System.Collections.Hashtable _hashSignificantValues;
        private System.Collections.ArrayList _histInfo;

        private UInt32 maxHisto;
        private string serverName;
        private string catalogName;
        private string modelName;
        private string treeName;
        private System.Drawing.Color colHighDensity;
        private System.Drawing.Color colLowDensity;
        private HistogramColorCollection colHistoColors;
        private System.Data.OleDb.OleDbConnection connection;

        public TreeViewerModel()
        {
            _histInfo = new ArrayList();
        }

        public uint TotalHistograms
        {
            get { return maxHisto; }
            set { maxHisto = value; }
        }

        public string ServerName
        {
            get { return serverName; }
            set { serverName = value; }
        }

        public string CatalogName
        {
            get { return catalogName; }
            set { catalogName = value; }
        }

        public string ModelName
        {
            get { return modelName; }
            set { modelName = value; }
        }

        public string TreeName
        {
            get { return treeName; }
            set { treeName = value; }
        }

        public System.Drawing.Color HighDensityColor
        {
            get { return colHighDensity; }
            set { colHighDensity = value; }
        }

        public System.Drawing.Color LowDensityColor
        {
            get { return colLowDensity; }
            set { colLowDensity = value; }
        }

        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA2227:CollectionPropertiesShouldBeReadOnly")]
        public HistogramColorCollection HistogramColorsCollection
        {
            get { return colHistoColors; }
            set { colHistoColors = value; }
        }

        public System.Data.OleDb.OleDbConnection DatabaseConnection
        {
            get { return connection; }
            set { connection = value; }
        }

        // Todo
        /// <summary>
        /// 
        /// </summary>
        /// <param name="nodeUniqueName"></param>
        /// <param name="arNodes"></param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Reliability", "CA2001:AvoidCallingProblematicMethods"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1045:DoNotPassTypesByReference")]
        public void GetSingleNode(string nodeUniqueName, ref System.Collections.ArrayList arNodes)
        {
            bool wasOpen = true;
            System.Data.OleDb.OleDbCommand cmd = null;
            System.Data.OleDb.OleDbDataReader rdr = null;

            try
            {
                if (connection == null)
                {
                    connection = new System.Data.OleDb.OleDbConnection();
                    connection.ConnectionString = "Provider=" + DMHtmlViewer.DMOlapProvider + "; Integrated Security=SSPI; Data Source=" + serverName + "; Initial Catalog=" + catalogName;
                }

                if (connection.State == System.Data.ConnectionState.Closed)
                {
                    wasOpen = false;
                    connection.Open();
                }

                // Prepare the has for significant values
                _hashSignificantValues = new System.Collections.Hashtable();
                _useCatchAllState = false;

                Hashtable hash = new Hashtable();

                _maxSupport = 0.0;

                cmd = new System.Data.OleDb.OleDbCommand();
                cmd.Connection = connection;

                // Get the root node
                cmd.CommandText = "select [PARENT_UNIQUE_NAME], NODE_UNIQUE_NAME, MSOLAP_NODE_SHORT_CAPTION, NODE_SUPPORT, ATTRIBUTE_NAME, NODE_DISTRIBUTION from [" +
                    modelName +
                    "].Content where node_unique_name = '" + nodeUniqueName + "'";

                getNodes(ref cmd, ref arNodes, ref hash);

                hash.Clear();
                _hashSignificantValues.Clear();

                recalTreeViewerNodeColors(ref arNodes);

                // Figure out whether it is continuous or not. Also, fetch min max if possible
                if (arNodes.Count > 0)
                {
                    TreeViewerNode node = (TreeViewerNode)arNodes[0];
                    node.IsRegression = false;

                    cmd.CommandText = "select distinct RangeMin([" + node.AttributeName + "]), " +
                                      " RangeMax([" + node.AttributeName + "]) from [" + modelName + "]";

					try
					{
						rdr = cmd.ExecuteReader();

						rdr.Read(); //skip empty line

						rdr.Read();
						node.ValueRangeMin = System.Convert.ToDouble(rdr.GetValue(0), CultureInfo.CurrentCulture);
						node.ValueRangeMax = System.Convert.ToDouble(rdr.GetValue(1), CultureInfo.CurrentCulture);
						node.IsRegression = true;
					}
					catch (System.Exception )
					{
						// Do nothing -- not a regression node
					}
                }
            }
            finally
            {
                if (cmd != null)
                {
                    cmd.Dispose();
                }
                if (rdr != null)
                {
                    if (!rdr.IsClosed)
                        rdr.Close();
                }
                if (!wasOpen)
                {
                    if (connection != null)
                    {
                        if (connection.State == ConnectionState.Open)
                        {
                            connection.Close();
                            connection.Dispose();
                        }
                    }
                }
                GC.Collect();
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="arNodes"></param>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Reliability", "CA2001:AvoidCallingProblematicMethods"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1045:DoNotPassTypesByReference")]
        public void GetContent(ref System.Collections.ArrayList arNodes)
        {
            bool wasOpen = true;
            System.Data.OleDb.OleDbCommand cmd = null;
            System.Data.OleDb.OleDbDataReader rdr = null;

            try
            {
                if (connection == null)
                {
                    connection = new System.Data.OleDb.OleDbConnection();
                    connection.ConnectionString = "Provider=" + DMHtmlViewer.DMOlapProvider + "; Integrated Security=SSPI; Data Source=" + serverName + "; Initial Catalog=" + catalogName;
                }

                if (connection.State == System.Data.ConnectionState.Closed)
                {
                    wasOpen = false;
                    connection.Open();
                }

                string escapedTreeName = treeName.Replace("'", "''");
                cmd = new System.Data.OleDb.OleDbCommand();
                cmd.CommandText = "select NODE_UNIQUE_NAME from [" + modelName + "].Content where node_type = 2 and attribute_name='" + escapedTreeName + "'";
                cmd.Connection = connection;

                rdr = cmd.ExecuteReader();

                string nodeUniqueName = "";

                if (rdr.Read())
                {
                    nodeUniqueName = rdr.GetString(0);
                }
                else
                {
                    return;
                }
                rdr.Close();

                // Prepare the has for significant values
                _hashSignificantValues = new System.Collections.Hashtable();
                _useCatchAllState = false;

                Hashtable hash = new Hashtable();

                _maxSupport = 0.0;

                // Get the root node
                cmd.CommandText = "select [PARENT_UNIQUE_NAME], NODE_UNIQUE_NAME, MSOLAP_NODE_SHORT_CAPTION, NODE_SUPPORT, ATTRIBUTE_NAME, NODE_DISTRIBUTION from [" +
                    modelName + "].Content where node_unique_name = '" + nodeUniqueName + "'";

                getNodes(ref cmd, ref arNodes, ref hash);

                // Get the root node
                cmd.CommandText = "select [PARENT_UNIQUE_NAME], NODE_UNIQUE_NAME, MSOLAP_NODE_SHORT_CAPTION, NODE_SUPPORT, ATTRIBUTE_NAME, NODE_DISTRIBUTION from [" +
                    modelName + "].Content where IsDescendant('" + nodeUniqueName + "')  order by node_unique_name";

                getNodes(ref cmd, ref arNodes, ref hash);

                hash.Clear();
                _hashSignificantValues.Clear();

                recalTreeViewerNodeColors(ref arNodes);
            }
            finally
            {
                if (cmd != null)
                {
                    cmd.Dispose();
                }
                if (rdr != null)
                {
                    if (!rdr.IsClosed)
                        rdr.Close();
                }
                if (!wasOpen)
                {
                    if (connection != null)
                    {
                        if (connection.State == ConnectionState.Open)
                        {
                            connection.Close();
                            connection.Dispose();
                        }
                    }
                }

                GC.Collect();
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1024:UsePropertiesWhereAppropriate")]
        public System.Collections.ArrayList GetHistogramsInfo()
        {
            return _histInfo;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="cmd"></param>
        /// <param name="arNodes"></param>
        /// <param name="hash"></param>
        private void getNodes(ref System.Data.OleDb.OleDbCommand cmd, ref System.Collections.ArrayList arNodes, ref System.Collections.Hashtable hash)
        {
            System.Data.OleDb.OleDbDataReader rdr = null;
            System.Data.OleDb.OleDbDataReader nestedRdr = null;

            try
            {
                rdr = cmd.ExecuteReader();

                while (rdr.Read())
                {
                    TreeViewerNode node = new TreeViewerNode();
                    node.TextColor = "#000000";
                    node.Caption = rdr.GetString(2);
                    node.Support = rdr.GetDouble(3);
                    node.AttributeName = rdr.GetString(4);

                    if (node.Support > _maxSupport)
                    {
                        _maxSupport = node.Support;
                    }

                    string uniqueName = rdr.GetString(0);
                    if (hash[uniqueName] != null)
                    {
                        node.ParentId = (int)hash[uniqueName];
                    }
                    else
                    {
                        node.ParentId = (int)-1;
                    }

                    uniqueName = rdr.GetString(1);
                    hash.Add(uniqueName, arNodes.Count);

                    string descriptionText;
                    descriptionText = "<font size='-1'>";
                    descriptionText += node.Caption + "<br>\r\n";
                    descriptionText += Properties.Resources.StrTotalCases + System.Convert.ToInt32(node.Support) + "<br>\r\n";
                    descriptionText += "<br>\r\n";

                    bool isRootNode = false;
                    isRootNode = (arNodes.Count == 0);

                    nestedRdr = (System.Data.OleDb.OleDbDataReader)rdr.GetValue(5);

                    if (!isRootNode)
                    {
                        // pre=allocate space for distributions
                        for (int i = 0; i < _hashSignificantValues.Count; i++)
                        {
                            node.DistributionsList.Add(null);
                        }
                    }

                    double dOtherSupport = 0.0;
                    while (nestedRdr.Read() /*&& iMaxDist < this.maxHisto*/)
                    {
                        Dist dist = new Dist();
                        dist.attributeName = nestedRdr.GetString(0); // attribute name
                        dist.name = nestedRdr.GetValue(1).ToString(); // attribute value
                        dist.attributeValue = nestedRdr.GetValue(1); // attribute value
                        dist.support = nestedRdr.GetDouble(2); // support
                        dist.probability = nestedRdr.GetDouble(3);
                        dist.histogram = System.Convert.ToInt32(100 * dist.probability);
                        dist.variance = nestedRdr.GetDouble(4);
						dist.valueType = nestedRdr.GetInt32(5);

						if ((dist.valueType == 8) || (dist.valueType == 9))
							continue;

                        // ATTRIBUTE_NAME, ATTRIBUTE_VALUE, SUPPORT, PROBABILITY,...
                        if (isRootNode)
                        {
                            // for the root node, add all the distributions
                            node.DistributionsList.Add(dist);
                        }
                        else
                        {
                            object objIndex = _hashSignificantValues[dist.name];
                            if (objIndex != null)
                            {
                                // If this attribute state is significant
                                node.DistributionsList[(int)objIndex] = dist;
                            }
                            else
                            {
                                // add the support to the [Other] state
                                dOtherSupport = dOtherSupport + dist.support;
                            }
                        }
                        descriptionText += dist.name + " " + System.Convert.ToInt32(dist.support, CultureInfo.CurrentCulture).ToString(CultureInfo.CurrentCulture);
                        descriptionText += "<br>\r\n";
                    }
                    nestedRdr.Close();

                    if (_useCatchAllState)
                    {
                        // add an Other distribution to the node
                        AddCatchAllStateToNode(ref node, dOtherSupport);
                    }

                    if (isRootNode)
                    {
                        // first, decide how many distributions we will use
                        if (node.DistributionsList.Count > this.maxHisto)
                        {
                            // We have too many states. Filter out the first (maxHisto-1), then add an [Other] state
                            DistComparer comp = new DistComparer();
                            node.DistributionsList.Sort(comp);


                            dOtherSupport = 0.0;
                            while (node.DistributionsList.Count > (this.maxHisto - 1))
                            {
                                dOtherSupport = dOtherSupport + ((Dist)node.DistributionsList[0]).support;
                                node.DistributionsList.RemoveAt(0);
                                _useCatchAllState = true;
                            }
                        }
                        // Now, add the interesting states to the hashSignificantValues so that the other nodes can filter them
                        for (int i = 0; i < node.DistributionsList.Count; i++)
                        {
                            string nameText = ((Dist)node.DistributionsList[i]).name;
                            if (_hashSignificantValues[nameText] == null)
                            {
                                _hashSignificantValues.Add(nameText, i);
                            }
                        }

                        if (_useCatchAllState)
                            AddCatchAllStateToNode(ref node, dOtherSupport);


                        // This is the first node
                        //BuildHistogramInfo(ref node.DistributionsList);
                        BuildHistogramInfo(node.DistributionsList);
                    }

                    node.Description = descriptionText;
                    arNodes.Add(node);
                }
            }
            catch (Exception)
            {
                // rethrow ex
                throw;
            }
            finally
            {
                if (rdr != null)
                {
                    if (!rdr.IsClosed)
                        rdr.Close();
                }
                if (nestedRdr != null)
                {
                    if (!nestedRdr.IsClosed)
                        nestedRdr.Close();
                }
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="node"></param>
        /// <param name="dOtherSupport"></param>
        private static void AddCatchAllStateToNode(ref TreeViewerNode node, double dOtherSupport)
        {
            Dist dist = new Dist();
            dist.name = Properties.Resources.StrAllOthers; ;
            dist.support = dOtherSupport; // support
            dist.probability = dOtherSupport / node.Support;
            dist.histogram = System.Convert.ToInt32(100 * dist.probability);
            node.DistributionsList.Add(dist);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="arDist"></param>
        private void BuildHistogramInfo(ArrayList arDist)
        {
            for (int i = 0; i < arDist.Count; i++)
            {
                Dist curDist = (Dist)arDist[i];
                HistogramInfo hist = new HistogramInfo();
                if (i < colHistoColors.Count)
                {
                    hist.Color = System.Drawing.ColorTranslator.ToHtml(colHistoColors[i]);
                }
                else
                {
                    hist.Color = System.Drawing.ColorTranslator.ToHtml(colHistoColors[i % colHistoColors.Count]);
                }

                hist.Name = curDist.name;
                hist.Index = i;
                _histInfo.Add(hist);
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="arNodes"></param>
        private void recalTreeViewerNodeColors(ref System.Collections.ArrayList arNodes)
        {
            double valueRangeMinR = colHighDensity.R, valueRangeMinG = colHighDensity.G, valueRangeMinB = colHighDensity.B;
            double valueRangeMaxR = colLowDensity.R, valueRangeMaxG = colLowDensity.G, valueRangeMaxB = colLowDensity.B;
            double dR, dG, dB;

            for (int i = 0; i < arNodes.Count; i++)
            {
                TreeViewerNode node = (TreeViewerNode)arNodes[i];
                double dRatio = node.Support / _maxSupport;

                dR = valueRangeMaxR - dRatio * (valueRangeMaxR - valueRangeMinR);
                dG = valueRangeMaxG - dRatio * (valueRangeMaxG - valueRangeMinG);
                dB = valueRangeMaxB - dRatio * (valueRangeMaxB - valueRangeMinB);

                System.Drawing.Color col = System.Drawing.Color.FromArgb(
                    System.Convert.ToInt32(dR),
                    System.Convert.ToInt32(dG),
                    System.Convert.ToInt32(dB));
                node.BackgroundColor = System.Drawing.ColorTranslator.ToHtml(col);
                if (col.GetBrightness() < 0.5)
                {
                    node.TextColor = "#ffffff";
                }
            }
        }
    }


    /// <summary>
    /// 
    /// </summary>
    public enum TreeOverflowBehaviorType
    {
        Visible,
        Scroll,
        Hidden,
        Auto
    };

    /// <summary>
    /// 
    /// </summary>
    public enum TreeTooltipEvent
    {
        OnMouseOverEvent,
        OnClickEvent
    };

    /// <summary>
    /// 
    /// </summary>
    public enum TreeViewType
    {
        Tree,
        NodeDetails,
    };
}